@@ -17,6 +17,7 @@
 
 from oslo_log import log as logging
 import six
+from six.moves import http_client
 import webob
 from webob import exc
 
@@ -24,9 +25,9 @@ from cinder.api import common
 from cinder.api import extensions
 from cinder.api.openstack import wsgi
 from cinder.api.views import cgsnapshots as cgsnapshot_views
-from cinder import consistencygroup as consistencygroupAPI
 from cinder import exception
-from cinder.i18n import _, _LI
+from cinder import group as group_api
+from cinder.i18n import _
 
 LOG = logging.getLogger(__name__)
 
@@ -37,7 +38,7 @@ class CgsnapshotsController(wsgi.Controller):
     _view_builder_class = cgsnapshot_views.ViewBuilder
 
     def __init__(self):
-        self.cgsnapshot_api = consistencygroupAPI.API()
+        self.group_snapshot_api = group_api.API()
         super(CgsnapshotsController, self).__init__()
 
     def show(self, req, id):
@@ -46,9 +47,7 @@ class CgsnapshotsController(wsgi.Controller):
         context = req.environ['cinder.context']
 
         # Not found exception will be handled at the wsgi level
-        cgsnapshot = self.cgsnapshot_api.get_cgsnapshot(
-            context,
-            cgsnapshot_id=id)
+        cgsnapshot = self._get_cgsnapshot(context, id)
 
         return self._view_builder.detail(req, cgsnapshot)
 
@@ -57,23 +56,21 @@ class CgsnapshotsController(wsgi.Controller):
         LOG.debug('delete called for member %s', id)
         context = req.environ['cinder.context']
 
-        LOG.info(_LI('Delete cgsnapshot with id: %s'), id)
-
+        LOG.info('Delete cgsnapshot with id: %s', id)
         try:
-            cgsnapshot = self.cgsnapshot_api.get_cgsnapshot(
-                context,
-                cgsnapshot_id=id)
-            self.cgsnapshot_api.delete_cgsnapshot(context, cgsnapshot)
-        except exception.CgSnapshotNotFound:
-            # Not found exception will be handled at the wsgi level
-            raise
-        except exception.InvalidCgSnapshot as e:
+            cgsnapshot = self._get_cgsnapshot(context, id)
+            self.group_snapshot_api.delete_group_snapshot(context, cgsnapshot)
+        except exception.InvalidGroupSnapshot as e:
             raise exc.HTTPBadRequest(explanation=six.text_type(e))
+        except (exception.GroupSnapshotNotFound,
+                exception.PolicyNotAuthorized) as e:
+            # Exceptions will be handled at the wsgi level
+            raise
         except Exception:
-            msg = _("Failed cgsnapshot")
+            msg = _('Failed to delete the cgsnapshot')
             raise exc.HTTPBadRequest(explanation=msg)
 
-        return webob.Response(status_int=202)
+        return webob.Response(status_int=http_client.ACCEPTED)
 
     def index(self, req):
         """Returns a summary list of cgsnapshots."""
@@ -83,19 +80,36 @@ class CgsnapshotsController(wsgi.Controller):
         """Returns a detailed list of cgsnapshots."""
         return self._get_cgsnapshots(req, is_detail=True)
 
+    def _get_cg(self, context, id):
+        # Not found exception will be handled at the wsgi level
+        consistencygroup = self.group_snapshot_api.get(context, group_id=id)
+
+        return consistencygroup
+
+    def _get_cgsnapshot(self, context, id):
+        # Not found exception will be handled at the wsgi level
+        cgsnapshot = self.group_snapshot_api.get_group_snapshot(
+            context, group_snapshot_id=id)
+
+        return cgsnapshot
+
     def _get_cgsnapshots(self, req, is_detail):
         """Returns a list of cgsnapshots, transformed through view builder."""
         context = req.environ['cinder.context']
-        cgsnapshots = self.cgsnapshot_api.get_all_cgsnapshots(context)
-        limited_list = common.limited(cgsnapshots, req)
+        grp_snapshots = self.group_snapshot_api.get_all_group_snapshots(
+            context)
+        grpsnap_limited_list = common.limited(grp_snapshots, req)
 
         if is_detail:
-            cgsnapshots = self._view_builder.detail_list(req, limited_list)
+            grp_snapshots = self._view_builder.detail_list(
+                req, grpsnap_limited_list)
         else:
-            cgsnapshots = self._view_builder.summary_list(req, limited_list)
-        return cgsnapshots
+            grp_snapshots = self._view_builder.summary_list(
+                req, grpsnap_limited_list)
 
-    @wsgi.response(202)
+        return grp_snapshots
+
+    @wsgi.response(http_client.ACCEPTED)
     def create(self, req, body):
         """Create a new cgsnapshot."""
         LOG.debug('Creating new cgsnapshot %s', body)
@@ -112,20 +126,22 @@ class CgsnapshotsController(wsgi.Controller):
             raise exc.HTTPBadRequest(explanation=msg)
 
         # Not found exception will be handled at the wsgi level
-        group = self.cgsnapshot_api.get(context, group_id)
+        group = self._get_cg(context, group_id)
 
         name = cgsnapshot.get('name', None)
         description = cgsnapshot.get('description', None)
 
-        LOG.info(_LI("Creating cgsnapshot %(name)s."),
+        LOG.info("Creating cgsnapshot %(name)s.",
                  {'name': name},
                  context=context)
 
         try:
-            new_cgsnapshot = self.cgsnapshot_api.create_cgsnapshot(
+            new_cgsnapshot = self.group_snapshot_api.create_group_snapshot(
                 context, group, name, description)
         # Not found exception will be handled at the wsgi level
-        except exception.InvalidCgSnapshot as error:
+        except (exception.InvalidGroup,
+                exception.InvalidGroupSnapshot,
+                exception.InvalidVolume) as error:
             raise exc.HTTPBadRequest(explanation=error.msg)
 
         retval = self._view_builder.summary(req, new_cgsnapshot)
