@@ -13,13 +13,13 @@
 #   under the License.
 
 from oslo_log import log as logging
+from six.moves import http_client
 import webob
 from webob import exc
 
 from cinder.api import extensions
 from cinder.api.openstack import wsgi
 from cinder import exception
-from cinder.i18n import _LI
 from cinder import volume
 
 LOG = logging.getLogger(__name__)
@@ -31,7 +31,7 @@ class SnapshotUnmanageController(wsgi.Controller):
         super(SnapshotUnmanageController, self).__init__(*args, **kwargs)
         self.volume_api = volume.API()
 
-    @wsgi.response(202)
+    @wsgi.response(http_client.ACCEPTED)
     @wsgi.action('os-unmanage')
     def unmanage(self, req, id, body):
         """Stop managing a snapshot.
@@ -48,17 +48,16 @@ class SnapshotUnmanageController(wsgi.Controller):
         context = req.environ['cinder.context']
         authorize(context)
 
-        LOG.info(_LI("Unmanage snapshot with id: %s"), id, context=context)
+        LOG.info("Unmanage snapshot with id: %s", id)
 
         try:
             snapshot = self.volume_api.get_snapshot(context, id)
             self.volume_api.delete_snapshot(context, snapshot,
                                             unmanage_only=True)
-        except exception.SnapshotNotFound as ex:
-            raise exc.HTTPNotFound(explanation=ex.msg)
+        # Not found exception will be handled at the wsgi level
         except exception.InvalidSnapshot as ex:
             raise exc.HTTPBadRequest(explanation=ex.msg)
-        return webob.Response(status_int=202)
+        return webob.Response(status_int=http_client.ACCEPTED)
 
 
 class Snapshot_unmanage(extensions.ExtensionDescriptor):
@@ -66,8 +65,6 @@ class Snapshot_unmanage(extensions.ExtensionDescriptor):
 
     name = "SnapshotUnmanage"
     alias = "os-snapshot-unmanage"
-    namespace = ('http://docs.openstack.org/snapshot/ext/snapshot-unmanage'
-                 '/api/v1')
     updated = "2014-12-31T00:00:00+00:00"
 
     def get_controller_extensions(self):
