@@ -19,8 +19,6 @@ import six
 from six.moves import http_client
 import webob
 
-from oslo_utils import strutils
-
 from cinder.api import extensions
 from cinder.api.openstack import wsgi
 from cinder.api.views import types as views_types
@@ -64,7 +62,8 @@ class VolumeTypesManageController(wsgi.Controller):
         description = vol_type.get('description')
         specs = vol_type.get('extra_specs', {})
         utils.validate_dictionary_string_length(specs)
-        is_public = vol_type.get('os-volume-type-access:is_public', True)
+        is_public = utils.get_bool_param('os-volume-type-access:is_public',
+                                         vol_type, True)
 
         if name is None or len(name.strip()) == 0:
             msg = _("Volume type name can not be empty.")
@@ -77,11 +76,6 @@ class VolumeTypesManageController(wsgi.Controller):
             utils.check_string_length(description, 'Type description',
                                       min_length=0, max_length=255)
 
-        if not strutils.is_valid_boolstr(is_public):
-            msg = _("Invalid value '%s' for is_public. Accepted values: "
-                    "True or False.") % is_public
-            raise webob.exc.HTTPBadRequest(explanation=msg)
-
         try:
             volume_types.create(context,
                                 name,
@@ -129,10 +123,8 @@ class VolumeTypesManageController(wsgi.Controller):
                     "a combination thereof.")
             raise webob.exc.HTTPBadRequest(explanation=msg)
 
-        if is_public is not None and not strutils.is_valid_boolstr(is_public):
-            msg = _("Invalid value '%s' for is_public. Accepted values: "
-                    "True or False.") % is_public
-            raise webob.exc.HTTPBadRequest(explanation=msg)
+        if is_public is not None:
+            is_public = utils.get_bool_param('is_public', vol_type)
 
         if name:
             utils.check_string_length(name, 'Type name',
