@@ -56,6 +56,11 @@ CONF = cfg.CONF
 CONF.register_opts(chunkedbackup_service_opts)
 
 
+# Object writer and reader returned by inheriting classes must not have any
+# logging calls, as well as the compression libraries, as eventlet has a bug
+# (https://github.com/eventlet/eventlet/issues/432) that would result in
+# failures.
+
 @six.add_metaclass(abc.ABCMeta)
 class ChunkedBackupDriver(driver.BackupDriver):
     """Abstract chunked backup driver.
@@ -76,12 +81,18 @@ class ChunkedBackupDriver(driver.BackupDriver):
         try:
             if algorithm.lower() in ('none', 'off', 'no'):
                 return None
-            elif algorithm.lower() in ('zlib', 'gzip'):
+            if algorithm.lower() in ('zlib', 'gzip'):
                 import zlib as compressor
-                return compressor
+                result = compressor
             elif algorithm.lower() in ('bz2', 'bzip2'):
                 import bz2 as compressor
-                return compressor
+                result = compressor
+            else:
+                result = None
+            if result:
+                # NOTE(geguileo): Compression/Decompression starves
+                # greenthreads so we use a native thread instead.
+                return eventlet.tpool.Proxy(result)
         except ImportError:
             pass
 
@@ -105,6 +116,16 @@ class ChunkedBackupDriver(driver.BackupDriver):
             self._get_compressor(CONF.backup_compression_algorithm)
         self.support_force_delete = True
 
+    def _get_object_writer(self, container, object_name, extra_metadata=None):
+        """Return writer proxy-wrapped to execute methods in native thread."""
+        writer = self.get_object_writer(container, object_name, extra_metadata)
+        return eventlet.tpool.Proxy(writer)
+
+    def _get_object_reader(self, container, object_name, extra_metadata=None):
+        """Return reader proxy-wrapped to execute methods in native thread."""
+        reader = self.get_object_reader(container, object_name, extra_metadata)
+        return eventlet.tpool.Proxy(reader)
+
     # To create your own "chunked" backup driver, implement the following
     # abstract methods.
 
@@ -122,14 +143,23 @@ class ChunkedBackupDriver(driver.BackupDriver):
     def get_object_writer(self, container, object_name, extra_metadata=None):
         """Returns a writer object which stores the chunk data in backup repository.
 
-           The object returned should be a context handler that can be used
-           in a "with" context.
+        The object returned should be a context handler that can be used in a
+        "with" context.
+
+        The object writer methods must not have any logging calls, as eventlet
+        has a bug (https://github.com/eventlet/eventlet/issues/432) that would
+        result in failures.
         """
         return
 
     @abc.abstractmethod
     def get_object_reader(self, container, object_name, extra_metadata=None):
-        """Returns a reader object for the backed up chunk."""
+        """Returns a reader object for the backed up chunk.
+
+        The object reader methods must not have any logging calls, as eventlet
+        has a bug (https://github.com/eventlet/eventlet/issues/432) that would
+        result in failures.
+        """
         return
 
     @abc.abstractmethod
@@ -222,7 +252,7 @@ class ChunkedBackupDriver(driver.BackupDriver):
         metadata_json = json.dumps(metadata, sort_keys=True, indent=2)
         if six.PY3:
             metadata_json = metadata_json.encode('utf-8')
-        with self.get_object_writer(container, filename) as writer:
+        with self._get_object_writer(container, filename) as writer:
             writer.write(metadata_json)
         LOG.debug('_write_metadata finished. Metadata: %s.', metadata_json)
 
@@ -243,7 +273,7 @@ class ChunkedBackupDriver(driver.BackupDriver):
         sha256file_json = json.dumps(sha256file, sort_keys=True, indent=2)
         if six.PY3:
             sha256file_json = sha256file_json.encode('utf-8')
-        with self.get_object_writer(container, filename) as writer:
+        with self._get_object_writer(container, filename) as writer:
             writer.write(sha256file_json)
         LOG.debug('_write_sha256file finished.')
 
@@ -253,7 +283,7 @@ class ChunkedBackupDriver(driver.BackupDriver):
         LOG.debug('_read_metadata started, container name: %(container)s, '
                   'metadata filename: %(filename)s.',
                   {'container': container, 'filename': filename})
-        with self.get_object_reader(container, filename) as reader:
+        with self._get_object_reader(container, filename) as reader:
             metadata_json = reader.read()
         if six.PY3:
             metadata_json = metadata_json.decode('utf-8')
@@ -267,7 +297,7 @@ class ChunkedBackupDriver(driver.BackupDriver):
         LOG.debug('_read_sha256file started, container name: %(container)s, '
                   'sha256 filename: %(filename)s.',
                   {'container': container, 'filename': filename})
-        with self.get_object_reader(container, filename) as reader:
+        with self._get_object_reader(container, filename) as reader:
             sha256file_json = reader.read()
         if six.PY3:
             sha256file_json = sha256file_json.decode('utf-8')
@@ -327,7 +357,7 @@ class ChunkedBackupDriver(driver.BackupDriver):
         algorithm, output_data = self._prepare_output_data(data)
         obj[object_name]['compression'] = algorithm
         LOG.debug('About to put_object')
-        with self.get_object_writer(
+        with self._get_object_writer(
                 container, object_name, extra_metadata=extra_metadata
         ) as writer:
             writer.write(output_data)
@@ -349,8 +379,7 @@ class ChunkedBackupDriver(driver.BackupDriver):
         data_size_bytes = len(data)
         # Execute compression in native thread so it doesn't prevent
         # cooperative greenthread switching.
-        compressed_data = eventlet.tpool.execute(self.compressor.compress,
-                                                 data)
+        compressed_data = self.compressor.compress(data)
         comp_size_bytes = len(compressed_data)
         algorithm = CONF.backup_compression_algorithm.lower()
         if comp_size_bytes >= data_size_bytes:
@@ -618,7 +647,7 @@ class ChunkedBackupDriver(driver.BackupDriver):
                           'volume_id': volume_id,
                       })
 
-            with self.get_object_reader(
+            with self._get_object_reader(
                     container, object_name,
                     extra_metadata=extra_metadata) as reader:
                 body = reader.read()
