@@ -32,7 +32,7 @@ import webob.exc
 from webob.util import status_generic_reasons
 from webob.util import status_reasons
 
-from cinder.i18n import _, _LE
+from cinder.i18n import _
 
 
 LOG = logging.getLogger(__name__)
@@ -98,6 +98,9 @@ class CinderException(Exception):
 
         for k, v in self.kwargs.items():
             if isinstance(v, Exception):
+                # NOTE(tommylikehu): If this is a cinder exception it will
+                # return the msg object, so we won't be preventing
+                # translations.
                 self.kwargs[k] = six.text_type(v)
 
         if self._should_format():
@@ -108,15 +111,18 @@ class CinderException(Exception):
                 exc_info = sys.exc_info()
                 # kwargs doesn't match a variable in the message
                 # log the issue and the kwargs
-                LOG.exception(_LE('Exception in string format operation'))
+                LOG.exception('Exception in string format operation')
                 for name, value in kwargs.items():
-                    LOG.error(_LE("%(name)s: %(value)s"),
+                    LOG.error("%(name)s: %(value)s",
                               {'name': name, 'value': value})
                 if CONF.fatal_exception_format_errors:
                     six.reraise(*exc_info)
                 # at least get the core message out if something happened
                 message = self.message
         elif isinstance(message, Exception):
+            # NOTE(tommylikehu): If this is a cinder exception it will
+            # return the msg object, so we won't be preventing
+            # translations.
             message = six.text_type(message)
 
         # NOTE(luisg): We put the actual message in 'msg' so that we can access
@@ -128,8 +134,12 @@ class CinderException(Exception):
     def _should_format(self):
         return self.kwargs['message'] is None or '%(message)' in self.message
 
+    # NOTE(tommylikehu): self.msg is already an unicode compatible object
+    # as the __init__ method ensures of it, and we should not be modifying
+    # it in any way with str(), unicode(), or six.text_type() as we would
+    # be preventing translations from happening.
     def __unicode__(self):
-        return six.text_type(self.msg)
+        return self.msg
 
 
 class VolumeBackendAPIException(CinderException):
@@ -200,6 +210,10 @@ class InvalidInput(Invalid):
     message = _("Invalid input received: %(reason)s")
 
 
+class InvalidAvailabilityZone(Invalid):
+    message = _("Availability zone '%(az)s' is invalid.")
+
+
 class InvalidVolumeType(Invalid):
     message = _("Invalid volume type: %(reason)s")
 
@@ -247,6 +261,11 @@ class ImageUnacceptable(Invalid):
     message = _("Image %(image_id)s is unacceptable: %(reason)s")
 
 
+class ImageTooBig(Invalid):
+    message = _("Image %(image_id)s size exceeded available "
+                "disk space: %(reason)s")
+
+
 class DeviceUnavailable(Invalid):
     message = _("The device in the path %(path)s is unavailable: %(reason)s")
 
@@ -256,7 +275,7 @@ class SnapshotUnavailable(VolumeBackendAPIException):
 
 
 class InvalidUUID(Invalid):
-    message = _("Expected a uuid but received %(uuid)s.")
+    message = _("Expected a UUID but received %(uuid)s.")
 
 
 class InvalidAPIVersionString(Invalid):
@@ -277,6 +296,10 @@ class MissingRequired(Invalid):
     message = _("Missing required element '%(element)s' in request body.")
 
 
+class ValidationError(Invalid):
+    message = "%(detail)s"
+
+
 class APIException(CinderException):
     message = _("Error while requesting %(service)s API.")
 
@@ -316,7 +339,7 @@ class MessageNotFound(NotFound):
 
 class VolumeAttachmentNotFound(NotFound):
     message = _("Volume attachment could not be found with "
-                "filter: %(filter)s .")
+                "filter: %(filter)s.")
 
 
 class VolumeMetadataNotFound(NotFound):
@@ -398,6 +421,10 @@ class ServerNotFound(NotFound):
     message = _("Instance %(uuid)s could not be found.")
 
 
+class VolumeSnapshotNotFound(NotFound):
+    message = _("No snapshots found for volume %(volume_id)s.")
+
+
 class VolumeIsBusy(CinderException):
     message = _("deleting volume %(volume_name)s that has snapshot")
 
@@ -730,6 +757,11 @@ class BackupMetadataUnsupportedVersion(BackupDriverException):
     message = _("Unsupported backup metadata version requested")
 
 
+class BackupMetadataNotFound(NotFound):
+    message = _("Backup %(backup_id)s has no metadata with "
+                "key %(metadata_key)s.")
+
+
 class BackupVerifyUnsupportedDriver(BackupDriverException):
     message = _("Unsupported backup verify driver")
 
@@ -842,6 +874,11 @@ class ReplicationError(CinderException):
                 "error: %(reason)s")
 
 
+class ReplicationGroupError(CinderException):
+    message = _("Group %(group_id)s replication "
+                "error: %(reason)s.")
+
+
 class ReplicationNotFound(NotFound):
     message = _("Volume replication for %(volume_id)s "
                 "could not be found.")
@@ -1190,6 +1227,10 @@ class BadHTTPResponseStatus(VolumeDriverException):
     message = _("Bad HTTP response status %(status)s")
 
 
+class BadResetResourceStatus(CinderException):
+    message = _("Bad reset resource status : %(message)s")
+
+
 # ZADARA STORAGE VPSA driver exception
 class ZadaraServerCreateFailure(VolumeDriverException):
     message = _("Unable to create server object for initiator %(name)s")
@@ -1266,6 +1307,10 @@ class DotHillNotTargetPortal(VolumeDriverException):
     message = _("No active iSCSI portals with supplied iSCSI IPs")
 
 
+class DotHillDriverNotSupported(VolumeDriverException):
+    message = _("The Dot Hill driver is no longer supported.")
+
+
 # Sheepdog
 class SheepdogError(VolumeBackendAPIException):
     message = _("An error has occurred in SheepdogDriver. "
@@ -1361,6 +1406,32 @@ class InvalidAttachment(Invalid):
     message = _("Invalid attachment: %(reason)s")
 
 
+# Veritas driver
+class UnableToExecuteHyperScaleCmd(VolumeDriverException):
+    message = _("Failed HyperScale command for '%(message)s'")
+
+
+class UnableToProcessHyperScaleCmdOutput(VolumeDriverException):
+    message = _("Failed processing command output '%(cmd_out)s'"
+                " for HyperScale command")
+
+
+class ErrorInFetchingConfiguration(VolumeDriverException):
+    message = _("Error in fetching configuration for '%(persona)s'")
+
+
+class ErrorInSendingMsg(VolumeDriverException):
+    message = _("Error in sending message '%(cmd_error)s'")
+
+
+class ErrorInHyperScaleVersion(VolumeDriverException):
+    message = _("Error in getting HyperScale version '%(cmd_error)s'")
+
+
+class ErrorInParsingArguments(VolumeDriverException):
+    message = _("Error in parsing message arguments : Invalid Payload")
+
+
 # GPFS driver
 class GPFSDriverUnsupportedOperation(VolumeBackendAPIException):
     message = _("GPFS driver unsupported operation: %(msg)s")
