@@ -408,8 +408,8 @@ def fetch_to_volume_format(context, image_service,
                 % {'fmt': fmt, 'backing_file': backing_file, })
 
         # NOTE(e0ne): check for free space in destination directory before
-        # image convertion.
-        check_available_space(dest, virt_size, image_id)
+        # image conversion.
+        check_available_space(dest, data.virtual_size, image_id)
 
         # NOTE(jdg): I'm using qemu-img convert to write
         # to the volume regardless if it *needs* conversion or not
