@@ -21,6 +21,7 @@ from cinder import exception
 from cinder.i18n import _
 from cinder import objects
 from cinder.objects import base
+from cinder.objects import cleanable
 from cinder.objects import fields as c_fields
 
 
@@ -28,11 +29,16 @@ CONF = cfg.CONF
 
 
 @base.CinderObjectRegistry.register
-class Snapshot(base.CinderPersistentObject, base.CinderObject,
-               base.CinderObjectDictCompat):
+class Snapshot(cleanable.CinderCleanableObject, base.CinderObject,
+               base.CinderObjectDictCompat, base.CinderComparableObject,
+               base.ClusteredObject):
     # Version 1.0: Initial version
     # Version 1.1: Changed 'status' field to use SnapshotStatusField
-    VERSION = '1.1'
+    # Version 1.2: This object is now cleanable (adds rows to workers table)
+    # Version 1.3: SnapshotStatusField now includes "unmanaging"
+    # Version 1.4: SnapshotStatusField now includes "backing-up"
+    # Version 1.5: SnapshotStatusField now includes "restoring"
+    VERSION = '1.5'
 
     # NOTE(thangp): OPTIONAL_FIELDS are fields that would be lazy-loaded. They
     # are typically the relationship in the sqlalchemy object.
@@ -68,8 +74,8 @@ class Snapshot(base.CinderPersistentObject, base.CinderObject,
     }
 
     @property
-    def service_topic_queue(self):
-        return self.volume.service_topic_queue
+    def cluster_name(self):
+        return self.volume.cluster_name
 
     @classmethod
     def _get_expected_attrs(cls, context, *args, **kwargs):
@@ -114,12 +120,27 @@ class Snapshot(base.CinderPersistentObject, base.CinderObject,
         super(Snapshot, self).obj_make_compatible(primitive, target_version)
         target_version = versionutils.convert_version_to_tuple(target_version)
 
-    @staticmethod
-    def _from_db_object(context, snapshot, db_snapshot, expected_attrs=None):
+        backport_statuses = (((1, 3),
+                              (c_fields.SnapshotStatus.UNMANAGING,
+                               c_fields.SnapshotStatus.DELETING)),
+                             ((1, 4),
+                             (c_fields.SnapshotStatus.BACKING_UP,
+                              c_fields.SnapshotStatus.AVAILABLE)),
+                             ((1, 5),
+                              (c_fields.SnapshotStatus.RESTORING,
+                               c_fields.SnapshotStatus.AVAILABLE)))
+        for version, status in backport_statuses:
+            if target_version < version:
+                if primitive.get('status') == status[0]:
+                    primitive['status'] = status[1]
+
+    @classmethod
+    def _from_db_object(cls, context, snapshot, db_snapshot,
+                        expected_attrs=None):
         if expected_attrs is None:
             expected_attrs = []
         for name, field in snapshot.fields.items():
-            if name in Snapshot.OPTIONAL_FIELDS:
+            if name in cls.OPTIONAL_FIELDS:
                 continue
             value = db_snapshot.get(name)
             if isinstance(field, fields.IntegerField):
@@ -130,12 +151,12 @@ class Snapshot(base.CinderPersistentObject, base.CinderObject,
             volume = objects.Volume(context)
             volume._from_db_object(context, volume, db_snapshot['volume'])
             snapshot.volume = volume
-        if 'cgsnapshot' in expected_attrs:
+        if snapshot.cgsnapshot_id and 'cgsnapshot' in expected_attrs:
             cgsnapshot = objects.CGSnapshot(context)
             cgsnapshot._from_db_object(context, cgsnapshot,
                                        db_snapshot['cgsnapshot'])
             snapshot.cgsnapshot = cgsnapshot
-        if 'group_snapshot' in expected_attrs:
+        if snapshot.group_snapshot_id and 'group_snapshot' in expected_attrs:
             group_snapshot = objects.GroupSnapshot(context)
             group_snapshot._from_db_object(context, group_snapshot,
                                            db_snapshot['group_snapshot'])
@@ -181,8 +202,11 @@ class Snapshot(base.CinderPersistentObject, base.CinderObject,
                 raise exception.ObjectActionError(action='save',
                                                   reason=_('volume changed'))
             if 'cgsnapshot' in updates:
-                raise exception.ObjectActionError(
-                    action='save', reason=_('cgsnapshot changed'))
+                # NOTE(xyang): Allow this to pass if 'cgsnapshot' is
+                # set to None. This is to support backward compatibility.
+                if updates.get('cgsnapshot'):
+                    raise exception.ObjectActionError(
+                        action='save', reason=_('cgsnapshot changed'))
             if 'group_snapshot' in updates:
                 raise exception.ObjectActionError(
                     action='save', reason=_('group_snapshot changed'))
@@ -204,7 +228,8 @@ class Snapshot(base.CinderPersistentObject, base.CinderObject,
         self.obj_reset_changes()
 
     def destroy(self):
-        updated_values = db.snapshot_destroy(self._context, self.id)
+        with self.obj_as_admin():
+            updated_values = db.snapshot_destroy(self._context, self.id)
         self.update(updated_values)
         self.obj_reset_changes(updated_values.keys())
 
@@ -222,13 +247,18 @@ class Snapshot(base.CinderPersistentObject, base.CinderObject,
                                                    self.volume_id)
 
         if attrname == 'cgsnapshot':
-            self.cgsnapshot = objects.CGSnapshot.get_by_id(self._context,
-                                                           self.cgsnapshot_id)
-
+            if self.cgsnapshot_id is None:
+                self.cgsnapshot = None
+            else:
+                self.cgsnapshot = objects.CGSnapshot.get_by_id(
+                    self._context, self.cgsnapshot_id)
         if attrname == 'group_snapshot':
-            self.group_snapshot = objects.GroupSnapshot.get_by_id(
-                self._context,
-                self.group_snapshot_id)
+            if self.group_snapshot_id is None:
+                self.group_snapshot = None
+            else:
+                self.group_snapshot = objects.GroupSnapshot.get_by_id(
+                    self._context,
+                    self.group_snapshot_id)
 
         self.obj_reset_changes(fields=[attrname])
 
@@ -248,6 +278,18 @@ class Snapshot(base.CinderPersistentObject, base.CinderObject,
         return db.snapshot_data_get_for_project(context, project_id,
                                                 volume_type_id)
 
+    @staticmethod
+    def _is_cleanable(status, obj_version):
+        # Before 1.2 we didn't have workers table, so cleanup wasn't supported.
+        if obj_version and obj_version < 1.2:
+            return False
+        return status == 'creating'
+
+    @property
+    def host(self):
+        """All cleanable VO must have a host property/attribute."""
+        return self.volume.host
+
 
 @base.CinderObjectRegistry.register
 class SnapshotList(base.ObjectListBase, base.CinderObject):
@@ -258,9 +300,14 @@ class SnapshotList(base.ObjectListBase, base.CinderObject):
     }
 
     @classmethod
-    def get_all(cls, context, search_opts, marker=None, limit=None,
+    def get_all(cls, context, filters, marker=None, limit=None,
                 sort_keys=None, sort_dirs=None, offset=None):
-        snapshots = db.snapshot_get_all(context, search_opts, marker, limit,
+        """Get all snapshot given some search_opts (filters).
+
+        Special filters accepted are host and cluster_name, that refer to the
+        volume's fields.
+        """
+        snapshots = db.snapshot_get_all(context, filters, marker, limit,
                                         sort_keys, sort_dirs, offset)
         expected_attrs = Snapshot._get_expected_attrs(context)
         return base.obj_make_list(context, cls(context), objects.Snapshot,
@@ -268,7 +315,7 @@ class SnapshotList(base.ObjectListBase, base.CinderObject):
 
     @classmethod
     def get_by_host(cls, context, host, filters=None):
-        snapshots = db.snapshot_get_by_host(context, host, filters)
+        snapshots = db.snapshot_get_all_by_host(context, host, filters)
         expected_attrs = Snapshot._get_expected_attrs(context)
         return base.obj_make_list(context, cls(context), objects.Snapshot,
                                   snapshots, expected_attrs=expected_attrs)
@@ -292,8 +339,8 @@ class SnapshotList(base.ObjectListBase, base.CinderObject):
                                   snapshots, expected_attrs=expected_attrs)
 
     @classmethod
-    def get_active_by_window(cls, context, begin, end):
-        snapshots = db.snapshot_get_active_by_window(context, begin, end)
+    def get_all_active_by_window(cls, context, begin, end):
+        snapshots = db.snapshot_get_all_active_by_window(context, begin, end)
         expected_attrs = Snapshot._get_expected_attrs(context)
         return base.obj_make_list(context, cls(context), objects.Snapshot,
                                   snapshots, expected_attrs=expected_attrs)
