@@ -13,7 +13,10 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from oslo_utils import uuidutils
+
 from cinder.tests.functional import functional_helpers
+from cinder.volume import configuration
 
 
 class VolumesTest(functional_helpers._FunctionalTestBase):
@@ -25,9 +28,10 @@ class VolumesTest(functional_helpers._FunctionalTestBase):
 
     def _get_flags(self):
         f = super(VolumesTest, self)._get_flags()
-        f['volume_driver'] = \
-            'cinder.tests.fake_driver.FakeLoggingVolumeDriver'
-        f['default_volume_type'] = self._vol_type_name
+        f['volume_driver'] = (
+            {'v': 'cinder.tests.fake_driver.FakeLoggingVolumeDriver',
+             'g': configuration.SHARED_CONF_GROUP})
+        f['default_volume_type'] = {'v': self._vol_type_name}
         return f
 
     def test_get_volumes_summary(self):
@@ -45,7 +49,7 @@ class VolumesTest(functional_helpers._FunctionalTestBase):
 
         # Create volume
         created_volume = self.api.post_volume({'volume': {'size': 1}})
-        self.assertTrue(created_volume['id'])
+        self.assertTrue(uuidutils.is_uuid_like(created_volume['id']))
         created_volume_id = created_volume['id']
 
         # Check it's there
@@ -82,7 +86,7 @@ class VolumesTest(functional_helpers._FunctionalTestBase):
         created_volume = self.api.post_volume(
             {'volume': {'size': 1,
                         'metadata': metadata}})
-        self.assertTrue(created_volume['id'])
+        self.assertTrue(uuidutils.is_uuid_like(created_volume['id']))
         created_volume_id = created_volume['id']
 
         # Check it's there and metadata present
@@ -98,7 +102,7 @@ class VolumesTest(functional_helpers._FunctionalTestBase):
         created_volume = self.api.post_volume(
             {'volume': {'size': 1,
                         'availability_zone': availability_zone}})
-        self.assertTrue(created_volume['id'])
+        self.assertTrue(uuidutils.is_uuid_like(created_volume['id']))
         created_volume_id = created_volume['id']
 
         # Check it's there and availability zone present
