@@ -23,38 +23,40 @@ from cinder import test
 # NOTE: The hashes in this list should only be changed if they come with a
 # corresponding version bump in the affected objects.
 object_data = {
-    'Backup': '1.4-c50f7a68bb4c400dd53dd219685b3992',
+    'Backup': '1.5-3ab4b305bd43ec0cff6701fe2a849194',
     'BackupDeviceInfo': '1.0-74b3950676c690538f4bc6796bd0042e',
-    'BackupImport': '1.4-c50f7a68bb4c400dd53dd219685b3992',
+    'BackupImport': '1.5-3ab4b305bd43ec0cff6701fe2a849194',
     'BackupList': '1.0-15ecf022a68ddbb8c2a6739cfc9f8f5e',
     'CleanupRequest': '1.0-e7c688b893e1d5537ccf65cc3eb10a28',
-    'Cluster': '1.1-cdb1572b250837933d950cc6662313b8',
+    'Cluster': '1.1-e2c533eb8cdd8d229b6c45c6cf3a9e2c',
     'ClusterList': '1.0-15ecf022a68ddbb8c2a6739cfc9f8f5e',
     'CGSnapshot': '1.1-3212ac2b4c2811b7134fb9ba2c49ff74',
     'CGSnapshotList': '1.0-15ecf022a68ddbb8c2a6739cfc9f8f5e',
     'ConsistencyGroup': '1.4-7bf01a79b82516639fc03cd3ab6d9c01',
     'ConsistencyGroupList': '1.1-15ecf022a68ddbb8c2a6739cfc9f8f5e',
+    'LogLevel': '1.0-7a8200b6b5063b33ec7b569dc6be66d2',
+    'LogLevelList': '1.0-15ecf022a68ddbb8c2a6739cfc9f8f5e',
     'ManageableSnapshot': '1.0-5be933366eb17d12db0115c597158d0d',
     'ManageableSnapshotList': '1.0-15ecf022a68ddbb8c2a6739cfc9f8f5e',
     'ManageableVolume': '1.0-5fd0152237ec9dfb7b5c7095b8b09ffa',
     'ManageableVolumeList': '1.0-15ecf022a68ddbb8c2a6739cfc9f8f5e',
     'QualityOfServiceSpecs': '1.0-0b212e0a86ee99092229874e03207fe8',
-    'QualityOfServiceSpecsList': '1.0-1b54e51ad0fc1f3a8878f5010e7e16dc',
+    'QualityOfServiceSpecsList': '1.0-15ecf022a68ddbb8c2a6739cfc9f8f5e',
     'RequestSpec': '1.1-b0bd1a28d191d75648901fa853e8a733',
-    'Service': '1.4-c7d011989d1718ca0496ccf640b42712',
+    'Service': '1.4-a6727ccda6d4043f5e38e75c7c518c7f',
     'ServiceList': '1.1-15ecf022a68ddbb8c2a6739cfc9f8f5e',
-    'Snapshot': '1.3-69dfbe3244992478a0174cb512cd7f27',
+    'Snapshot': '1.5-ac1cdbd5b89588f6a8f44afdf6b8b201',
     'SnapshotList': '1.0-15ecf022a68ddbb8c2a6739cfc9f8f5e',
     'Volume': '1.6-7d3bc8577839d5725670d55e480fe95f',
     'VolumeList': '1.1-15ecf022a68ddbb8c2a6739cfc9f8f5e',
-    'VolumeAttachment': '1.1-ed82a5fdd56655e14d9f86396c130aea',
+    'VolumeAttachment': '1.2-b68b357a1756582b706006ea9de40c9a',
     'VolumeAttachmentList': '1.1-15ecf022a68ddbb8c2a6739cfc9f8f5e',
     'VolumeProperties': '1.1-cadac86b2bdc11eb79d1dcea988ff9e8',
     'VolumeType': '1.3-a5d8c3473db9bc3bbcdbab9313acf4d1',
     'VolumeTypeList': '1.1-15ecf022a68ddbb8c2a6739cfc9f8f5e',
     'GroupType': '1.0-d4a7b272199d0b0d6fc3ceed58539d30',
-    'GroupTypeList': '1.0-1b54e51ad0fc1f3a8878f5010e7e16dc',
-    'Group': '1.1-bd853b1d1ee05949d9ce4b33f80ac1a0',
+    'GroupTypeList': '1.0-15ecf022a68ddbb8c2a6739cfc9f8f5e',
+    'Group': '1.2-2ade6acf2e55687b980048fc3f51dad9',
     'GroupList': '1.0-15ecf022a68ddbb8c2a6739cfc9f8f5e',
     'GroupSnapshot': '1.0-9af3e994e889cbeae4427c3e351fa91d',
     'GroupSnapshotList': '1.0-15ecf022a68ddbb8c2a6739cfc9f8f5e',
@@ -68,9 +70,14 @@ class TestObjectVersions(test.TestCase):
             base.CinderObjectRegistry.obj_classes())
         expected, actual = checker.test_hashes(object_data)
         self.assertEqual(expected, actual,
-                         'Some objects have changed; please make sure the '
-                         'versions have been bumped, and then update their '
-                         'hashes in the object_data map in this test module.')
+                         "Some objects have changed; please make sure the "
+                         "versions have been bumped and backporting "
+                         "compatibility code has been added to "
+                         "obj_make_compatible if necessary, and then update "
+                         "their hashes in the object_data map in this test "
+                         "module.  If we don't need to add backporting code "
+                         "then it means we also don't need the version bump "
+                         "and we just have to change the hash in this module.")
 
     def test_versions_history(self):
         classes = base.CinderObjectRegistry.obj_classes()
@@ -119,7 +126,9 @@ class TestObjectVersions(test.TestCase):
         # the converted data, but at least ensures the method doesn't blow
         # up on something simple.
         init_args = {}
-        init_kwargs = {objects.Snapshot: {'context': 'ctxt'}}
+        init_kwargs = {objects.Snapshot: {'context': 'ctxt'},
+                       objects.Backup: {'context': 'ctxt'},
+                       objects.BackupImport: {'context': 'ctxt'}}
         checker = fixture.ObjectVersionChecker(
             base.CinderObjectRegistry.obj_classes())
         checker.test_compatibility_routines(init_args=init_args,
