@@ -46,7 +46,8 @@ class FakeHostManager(host_manager.HostManager):
                       'thick_provisioning_support': True,
                       'reserved_percentage': 10,
                       'volume_backend_name': 'lvm1',
-                      'timestamp': UTC_NOW},
+                      'timestamp': UTC_NOW,
+                      'multiattach': True},
             'host2': {'total_capacity_gb': 2048,
                       'free_capacity_gb': 300,
                       'allocated_capacity_gb': 1748,
@@ -77,7 +78,7 @@ class FakeHostManager(host_manager.HostManager):
                       'reserved_percentage': 5,
                       'volume_backend_name': 'lvm4',
                       'timestamp': UTC_NOW,
-                      'consistencygroup_support': True},
+                      'consistent_group_snapshot_enabled': True},
             'host5': {'total_capacity_gb': 'infinite',
                       'free_capacity_gb': 'unknown',
                       'allocated_capacity_gb': 1548,
