@@ -13,12 +13,14 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import ddt
 import mock
 
 from cinder import context
 from cinder import test
 
 
+@ddt.ddt
 class ContextTestCase(test.TestCase):
 
     def test_request_context_sets_is_admin(self):
@@ -58,13 +60,13 @@ class ContextTestCase(test.TestCase):
 
     def test_request_context_elevated(self):
         user_context = context.RequestContext(
-            'fake_user', 'fake_project', admin=False)
+            'fake_user', 'fake_project', is_admin=False)
         self.assertFalse(user_context.is_admin)
         admin_context = user_context.elevated()
         self.assertFalse(user_context.is_admin)
         self.assertTrue(admin_context.is_admin)
-        self.assertFalse('admin' in user_context.roles)
-        self.assertTrue('admin' in admin_context.roles)
+        self.assertNotIn('admin', user_context.roles)
+        self.assertIn('admin', admin_context.roles)
 
     def test_service_catalog_nova_and_swift(self):
         service_catalog = [
@@ -82,7 +84,7 @@ class ContextTestCase(test.TestCase):
         object_catalog = [{u'name': u'swift', u'type': u'object-store'}]
         ctxt = context.RequestContext('111', '222',
                                       service_catalog=service_catalog)
-        self.assertEqual(3, len(ctxt.service_catalog))
+        self.assertEqual(4, len(ctxt.service_catalog))
         return_compute = [v for v in ctxt.service_catalog if
                           v['type'] == u'compute']
         return_object = [v for v in ctxt.service_catalog if
@@ -98,39 +100,31 @@ class ContextTestCase(test.TestCase):
         self.assertEqual('user tenant domain user-domain project-domain',
                          ctx.to_dict()["user_identity"])
 
+    @ddt.data(('ec729e9946bc43c39ece6dfa7de70eea',
+               'c466a48309794261b64a4f02cfcc3d64'),
+              ('ec729e9946bc43c39ece6dfa7de70eea', None),
+              (None, 'c466a48309794261b64a4f02cfcc3d64'),
+              (None, None))
+    @ddt.unpack
     @mock.patch('cinder.context.CONF')
-    def test_cinder_internal_context(self, mock_conf):
-        project_id = 'ec729e9946bc43c39ece6dfa7de70eea'
-        user_id = 'c466a48309794261b64a4f02cfcc3d64'
+    def test_cinder_internal_context(self, project_id, user_id, mock_conf):
         mock_conf.cinder_internal_tenant_project_id = project_id
         mock_conf.cinder_internal_tenant_user_id = user_id
         ctx = context.get_internal_tenant_context()
-        self.assertEqual(user_id, ctx.user_id)
-        self.assertEqual(project_id, ctx.project_id)
+        if project_id is None or user_id is None:
+            self.assertIsNone(ctx)
+        else:
+            self.assertEqual(user_id, ctx.user_id)
+            self.assertEqual(project_id, ctx.project_id)
 
-    @mock.patch('cinder.context.CONF')
-    def test_cinder_internal_context_missing_user(self, mock_conf):
-        project_id = 'ec729e9946bc43c39ece6dfa7de70eea'
-        user_id = None
-        mock_conf.cinder_internal_tenant_project_id = project_id
-        mock_conf.cinder_internal_tenant_user_id = user_id
-        ctx = context.get_internal_tenant_context()
-        self.assertIsNone(ctx)
+    def test_request_context_no_roles(self):
+        ctxt = context.RequestContext('111',
+                                      '222')
+        self.assertEqual([], ctxt.roles)
 
-    @mock.patch('cinder.context.CONF')
-    def test_cinder_internal_context_missing_project(self, mock_conf):
-        project_id = None
-        user_id = 'c466a48309794261b64a4f02cfcc3d64'
-        mock_conf.cinder_internal_tenant_project_id = project_id
-        mock_conf.cinder_internal_tenant_user_id = user_id
-        ctx = context.get_internal_tenant_context()
-        self.assertIsNone(ctx)
-
-    @mock.patch('cinder.context.CONF')
-    def test_cinder_internal_context_missing_all(self, mock_conf):
-        project_id = None
-        user_id = None
-        mock_conf.cinder_internal_tenant_project_id = project_id
-        mock_conf.cinder_internal_tenant_user_id = user_id
-        ctx = context.get_internal_tenant_context()
-        self.assertIsNone(ctx)
+    def test_request_context_with_roles(self):
+        roles = ['alpha', 'beta']
+        ctxt = context.RequestContext('111',
+                                      '222',
+                                      roles=roles)
+        self.assertEqual(roles, ctxt.roles)
