@@ -919,11 +919,6 @@ class NetAppNfsDriverTestCase(test.TestCase):
                           volume,
                           vol_ref)
 
-    def test_create_consistency_group(self):
-        model_update = self.driver.create_consistencygroup(
-            fake.CG_CONTEXT, fake.CONSISTENCY_GROUP)
-        self.assertEqual('available', model_update['status'])
-
     @ddt.data(True, False)
     def test_delete_file(self, volume_not_present):
         mock_get_provider_location = self.mock_object(
@@ -966,160 +961,6 @@ class NetAppNfsDriverTestCase(test.TestCase):
         mock_get_volume_path.assert_not_called()
         mock_delete.assert_not_called()
 
-    def test_update_consistencygroup(self):
-        model_update, add_volumes_update, remove_volumes_update = (
-            self.driver.update_consistencygroup(fake.CG_CONTEXT, "foo"))
-        self.assertIsNone(add_volumes_update)
-        self.assertIsNone(remove_volumes_update)
-
-    @ddt.data(None,
-              {'replication_status': fields.ReplicationStatus.ENABLED})
-    def test_create_consistencygroup_from_src(self, volume_model_update):
-        volume_model_update = volume_model_update or {}
-        volume_model_update.update(
-            {'provider_location': fake.PROVIDER_LOCATION})
-        mock_create_volume_from_snapshot = self.mock_object(
-            self.driver, 'create_volume_from_snapshot',
-            return_value=volume_model_update)
-
-        model_update, volumes_model_update = (
-            self.driver.create_consistencygroup_from_src(
-                fake.CG_CONTEXT, fake.CONSISTENCY_GROUP, [fake.VOLUME],
-                cgsnapshot=fake.CG_SNAPSHOT, snapshots=[fake.SNAPSHOT]))
-
-        expected_volumes_model_updates = [{'id': fake.VOLUME['id']}]
-        expected_volumes_model_updates[0].update(volume_model_update)
-        mock_create_volume_from_snapshot.assert_called_once_with(
-            fake.VOLUME, fake.SNAPSHOT)
-        self.assertIsNone(model_update)
-        self.assertEqual(expected_volumes_model_updates, volumes_model_update)
-
-    @ddt.data(None,
-              {'replication_status': fields.ReplicationStatus.ENABLED})
-    def test_create_consistencygroup_from_src_source_vols(
-            self, volume_model_update):
-        mock_get_snapshot_flexvols = self.mock_object(
-            self.driver, '_get_flexvol_names_from_hosts')
-        mock_get_snapshot_flexvols.return_value = (set([fake.CG_POOL_NAME]))
-        mock_clone_backing_file = self.mock_object(
-            self.driver, '_clone_backing_file_for_volume')
-        fake_snapshot_name = 'snapshot-temp-' + fake.CONSISTENCY_GROUP['id']
-        mock_busy = self.mock_object(
-            self.driver.zapi_client, 'wait_for_busy_snapshot')
-        self.mock_object(self.driver, '_get_volume_model_update',
-                         return_value=volume_model_update)
-
-        model_update, volumes_model_update = (
-            self.driver.create_consistencygroup_from_src(
-                fake.CG_CONTEXT, fake.CONSISTENCY_GROUP, [fake.VOLUME],
-                source_cg=fake.CONSISTENCY_GROUP,
-                source_vols=[fake.NFS_VOLUME]))
-
-        expected_volumes_model_updates = [{
-            'id': fake.NFS_VOLUME['id'],
-            'provider_location': fake.PROVIDER_LOCATION,
-        }]
-        if volume_model_update:
-            expected_volumes_model_updates[0].update(volume_model_update)
-        mock_get_snapshot_flexvols.assert_called_once_with(
-            [fake.NFS_VOLUME['host']])
-        self.driver.zapi_client.create_cg_snapshot.assert_called_once_with(
-            set([fake.CG_POOL_NAME]), fake_snapshot_name)
-        mock_clone_backing_file.assert_called_once_with(
-            fake.NFS_VOLUME['name'], fake.VOLUME['name'],
-            fake.NFS_VOLUME['id'], source_snapshot=fake_snapshot_name)
-        mock_busy.assert_called_once_with(
-            fake.CG_POOL_NAME, fake_snapshot_name)
-        self.driver.zapi_client.delete_snapshot.assert_called_once_with(
-            fake.CG_POOL_NAME, fake_snapshot_name)
-        self.assertIsNone(model_update)
-        self.assertEqual(expected_volumes_model_updates, volumes_model_update)
-
-    def test_create_consistencygroup_from_src_invalid_parms(self):
-
-        model_update, volumes_model_update = (
-            self.driver.create_consistencygroup_from_src(
-                fake.CG_CONTEXT, fake.CONSISTENCY_GROUP, [fake.VOLUME]))
-
-        self.assertIn('error', model_update['status'])
-
-    def test_create_cgsnapshot(self):
-        snapshot = fake.CG_SNAPSHOT
-        snapshot['volume'] = fake.CG_VOLUME
-        mock_get_snapshot_flexvols = self.mock_object(
-            self.driver, '_get_flexvol_names_from_hosts')
-        mock_get_snapshot_flexvols.return_value = (set([fake.CG_POOL_NAME]))
-        mock_clone_backing_file = self.mock_object(
-            self.driver, '_clone_backing_file_for_volume')
-        mock_busy = self.mock_object(
-            self.driver.zapi_client, 'wait_for_busy_snapshot')
-
-        self.driver.create_cgsnapshot(
-            fake.CG_CONTEXT, fake.CG_SNAPSHOT, [snapshot])
-
-        mock_get_snapshot_flexvols.assert_called_once_with(
-            [snapshot['volume']['host']])
-        self.driver.zapi_client.create_cg_snapshot.assert_called_once_with(
-            set([fake.CG_POOL_NAME]), fake.CG_SNAPSHOT_ID)
-        mock_clone_backing_file.assert_called_once_with(
-            snapshot['volume']['name'], snapshot['name'],
-            snapshot['volume']['id'], source_snapshot=fake.CG_SNAPSHOT_ID)
-        mock_busy.assert_called_once_with(
-            fake.CG_POOL_NAME, fake.CG_SNAPSHOT_ID)
-        self.driver.zapi_client.delete_snapshot.assert_called_once_with(
-            fake.CG_POOL_NAME, fake.CG_SNAPSHOT_ID)
-
-    def test_create_cgsnapshot_busy_snapshot(self):
-        snapshot = fake.CG_SNAPSHOT
-        snapshot['volume'] = fake.CG_VOLUME
-        mock_get_snapshot_flexvols = self.mock_object(
-            self.driver, '_get_flexvol_names_from_hosts')
-        mock_get_snapshot_flexvols.return_value = (set([fake.CG_POOL_NAME]))
-        mock_clone_backing_file = self.mock_object(
-            self.driver, '_clone_backing_file_for_volume')
-        mock_busy = self.mock_object(
-            self.driver.zapi_client, 'wait_for_busy_snapshot')
-        mock_busy.side_effect = exception.SnapshotIsBusy(snapshot['name'])
-        mock_mark_snapshot_for_deletion = self.mock_object(
-            self.zapi_client, 'mark_snapshot_for_deletion')
-
-        self.driver.create_cgsnapshot(
-            fake.CG_CONTEXT, fake.CG_SNAPSHOT, [snapshot])
-
-        mock_get_snapshot_flexvols.assert_called_once_with(
-            [snapshot['volume']['host']])
-        self.driver.zapi_client.create_cg_snapshot.assert_called_once_with(
-            set([fake.CG_POOL_NAME]), fake.CG_SNAPSHOT_ID)
-        mock_clone_backing_file.assert_called_once_with(
-            snapshot['volume']['name'], snapshot['name'],
-            snapshot['volume']['id'], source_snapshot=fake.CG_SNAPSHOT_ID)
-        mock_busy.assert_called_once_with(
-            fake.CG_POOL_NAME, fake.CG_SNAPSHOT_ID)
-        self.driver.zapi_client.delete_snapshot.assert_not_called()
-        mock_mark_snapshot_for_deletion.assert_called_once_with(
-            fake.CG_POOL_NAME, fake.CG_SNAPSHOT_ID)
-
-    def test_delete_consistencygroup_volume_delete_failure(self):
-        self.mock_object(self.driver, '_delete_file', side_effect=Exception)
-
-        model_update, volumes = self.driver.delete_consistencygroup(
-            fake.CG_CONTEXT, fake.CONSISTENCY_GROUP, [fake.CG_VOLUME])
-
-        self.assertEqual('deleted', model_update['status'])
-        self.assertEqual('error_deleting', volumes[0]['status'])
-
-    def test_delete_consistencygroup(self):
-        mock_delete_file = self.mock_object(
-            self.driver, '_delete_file')
-
-        model_update, volumes = self.driver.delete_consistencygroup(
-            fake.CG_CONTEXT, fake.CONSISTENCY_GROUP, [fake.CG_VOLUME])
-
-        self.assertEqual('deleted', model_update['status'])
-        self.assertEqual('deleted', volumes[0]['status'])
-        mock_delete_file.assert_called_once_with(
-            fake.CG_VOLUME_ID, fake.CG_VOLUME_NAME)
-
     def test_check_for_setup_error(self):
         super_check_for_setup_error = self.mock_object(
             nfs.NfsDriver, 'check_for_setup_error')
@@ -1145,24 +986,41 @@ class NetAppNfsDriverTestCase(test.TestCase):
                       loopingcalls.ONE_MINUTE),
             mock.call(mock_call_ems_logging, loopingcalls.ONE_HOUR)])
 
-    def test_delete_snapshots_marked_for_deletion(self):
-        snapshots = [{
-            'name': fake.SNAPSHOT_NAME,
-            'volume_name': fake.VOLUME['name']
-        }]
-        mock_get_flexvol_names = self.mock_object(
-            self.driver, '_get_backing_flexvol_names')
-        mock_get_flexvol_names.return_value = [fake.VOLUME['name']]
-        mock_get_snapshots_marked = self.mock_object(
-            self.zapi_client, 'get_snapshots_marked_for_deletion')
-        mock_get_snapshots_marked.return_value = snapshots
-        mock_delete_snapshot = self.mock_object(
-            self.zapi_client, 'delete_snapshot')
+    def test__clone_from_cache(self):
+        image_id = 'fake_image_id'
+        cache_result = [
+            ('fakepool_bad1', '/fakepath/img-cache-1'),
+            ('fakepool', '/fakepath/img-cache-2'),
+            ('fakepool_bad2', '/fakepath/img-cache-3'),
+        ]
+        mock_call__is_share_clone_compatible = self.mock_object(
+            self.driver, '_is_share_clone_compatible')
+        mock_call__is_share_clone_compatible.return_value = True
+        mock_call__do_clone_rel_img_cache = self.mock_object(
+            self.driver, '_do_clone_rel_img_cache')
+        cloned = self.driver._clone_from_cache(fake.test_volume, image_id,
+                                               cache_result)
+        self.assertTrue(cloned)
+        mock_call__is_share_clone_compatible.assert_called_once_with(
+            fake.test_volume, 'fakepool')
+        mock_call__do_clone_rel_img_cache.assert_called_once_with(
+            '/fakepath/img-cache-2', 'fakename', 'fakepool',
+            '/fakepath/img-cache-2'
+        )
 
-        self.driver._delete_snapshots_marked_for_deletion()
-
-        mock_get_flexvol_names.assert_called_once_with()
-        mock_get_snapshots_marked.assert_called_once_with(
-            [fake.VOLUME['name']])
-        mock_delete_snapshot.assert_called_once_with(
-            fake.VOLUME['name'], fake.SNAPSHOT_NAME)
+    def test__clone_from_cache_not_found(self):
+        image_id = 'fake_image_id'
+        cache_result = [
+            ('fakepool_bad1', '/fakepath/img-cache-1'),
+            ('fakepool_bad2', '/fakepath/img-cache-2'),
+            ('fakepool_bad3', '/fakepath/img-cache-3'),
+        ]
+        mock_call__is_share_clone_compatible = self.mock_object(
+            self.driver, '_is_share_clone_compatible')
+        mock_call__do_clone_rel_img_cache = self.mock_object(
+            self.driver, '_do_clone_rel_img_cache')
+        cloned = self.driver._clone_from_cache(fake.test_volume, image_id,
+                                               cache_result)
+        self.assertFalse(cloned)
+        mock_call__is_share_clone_compatible.assert_not_called()
+        mock_call__do_clone_rel_img_cache.assert_not_called()
