@@ -867,10 +867,13 @@ class API(base.Base):
             msg = _("Snapshot of secondary replica is not allowed.")
             raise exception.InvalidVolume(reason=msg)
 
-        if ((not force) and (volume['status'] != "available")):
-            msg = _("Volume %(vol_id)s status must be available, "
+        valid_status = ["available", "in-use"] if force else ["available"]
+
+        if volume['status'] not in valid_status:
+            msg = _("Volume %(vol_id)s status must be %(status)s, "
                     "but current status is: "
                     "%(vol_status)s.") % {'vol_id': volume['id'],
+                                          'status': ', '.join(valid_status),
                                           'vol_status': volume['status']}
             raise exception.InvalidVolume(reason=msg)
 
@@ -911,7 +914,17 @@ class API(base.Base):
             }
             snapshot = objects.Snapshot(context=context, **kwargs)
             snapshot.create()
+            volume.refresh()
 
+            if volume['status'] not in valid_status:
+                msg = _("Volume %(vol_id)s status must be %(status)s , "
+                        "but current status is: "
+                        "%(vol_status)s.") % {'vol_id': volume['id'],
+                                              'status':
+                                                  ', '.join(valid_status),
+                                              'vol_status':
+                                                  volume['status']}
+                raise exception.InvalidVolume(reason=msg)
             if commit_quota:
                 QUOTAS.commit(context, reservations)
         except Exception:
@@ -1703,11 +1716,16 @@ class API(base.Base):
         if availability_zone is None:
             availability_zone = service.availability_zone
 
+        if not cluster_name and bool(volume_utils.extract_host(host, 'pool')):
+            manage_host = host
+        else:
+            manage_host = service.host
+
         manage_what = {
             'context': context,
             'name': name,
             'description': description,
-            'host': service.host,
+            'host': manage_host,
             'cluster_name': service.cluster_name,
             'ref': ref,
             'volume_type': volume_type,
