@@ -29,6 +29,8 @@ from six.moves import urllib
 from cinder import context
 from cinder import exception
 from cinder.i18n import _
+from cinder import interface
+from cinder.volume import configuration
 from cinder.volume import driver
 from cinder.volume import utils as volume_utils
 
@@ -36,33 +38,33 @@ LOG = logging.getLogger(__name__)
 
 blockbridge_opts = [
     cfg.StrOpt("blockbridge_api_host",
-               help=_("IP address/hostname of Blockbridge API.")),
+               help="IP address/hostname of Blockbridge API."),
     cfg.IntOpt("blockbridge_api_port",
-               help=_("Override HTTPS port to connect to Blockbridge "
-                      "API server.")),
+               help="Override HTTPS port to connect to Blockbridge "
+                    "API server."),
     cfg.StrOpt("blockbridge_auth_scheme",
                default='token',
                choices=['token', 'password'],
-               help=_("Blockbridge API authentication scheme (token "
-                      "or password)")),
+               help="Blockbridge API authentication scheme (token "
+                    "or password)"),
     cfg.StrOpt("blockbridge_auth_token",
-               help=_("Blockbridge API token (for auth scheme 'token')"),
+               help="Blockbridge API token (for auth scheme 'token')",
                secret=True),
     cfg.StrOpt("blockbridge_auth_user",
-               help=_("Blockbridge API user (for auth scheme 'password')")),
+               help="Blockbridge API user (for auth scheme 'password')"),
     cfg.StrOpt("blockbridge_auth_password",
-               help=_("Blockbridge API password (for auth scheme 'password')"),
+               help="Blockbridge API password (for auth scheme 'password')",
                secret=True),
     cfg.DictOpt("blockbridge_pools",
                 default={'OpenStack': '+openstack'},
-                help=_("Defines the set of exposed pools and their associated "
-                       "backend query strings")),
+                help="Defines the set of exposed pools and their associated "
+                     "backend query strings"),
     cfg.StrOpt("blockbridge_default_pool",
-               help=_("Default pool name if unspecified.")),
+               help="Default pool name if unspecified."),
 ]
 
 CONF = cfg.CONF
-CONF.register_opts(blockbridge_opts)
+CONF.register_opts(blockbridge_opts, group=configuration.SHARED_CONF_GROUP)
 
 
 class BlockbridgeAPIClient(object):
@@ -168,11 +170,19 @@ class BlockbridgeAPIClient(object):
         return rsp_data
 
 
+@interface.volumedriver
 class BlockbridgeISCSIDriver(driver.ISCSIDriver):
     """Manages volumes hosted on Blockbridge EPS."""
 
     VERSION = '1.3.0'
 
+    # ThirdPartySystems wiki page
+    CI_WIKI_NAME = "Blockbridge_EPS_CI"
+
+    # TODO(smcginnis) Either remove this if CI requirements are met, or
+    # remove this driver in the Queens release per normal deprecation
+    SUPPORTED = False
+
     def __init__(self, *args, **kwargs):
         super(BlockbridgeISCSIDriver, self).__init__(*args, **kwargs)
 
