@@ -38,15 +38,18 @@ import string
 
 from oslo_config import cfg
 from oslo_log import log as logging
+from oslo_utils import strutils
 from oslo_utils import units
 import six
+from six.moves import http_client
 
 from cinder import context
 from cinder import exception
-from cinder.i18n import _, _LE, _LI, _LW
+from cinder.i18n import _
 from cinder import interface
 from cinder.objects import fields
 from cinder import utils
+from cinder.volume import configuration
 from cinder.volume import driver
 from cinder.volume.drivers.san import san
 from cinder.zonemanager import utils as fczm_utils
@@ -70,7 +73,7 @@ XTREMIO_OPTS = [
                default=100,
                help='Number of volumes created from each cached glance image')]
 
-CONF.register_opts(XTREMIO_OPTS)
+CONF.register_opts(XTREMIO_OPTS, group=configuration.SHARED_CONF_GROUP)
 
 RANDOM = random.Random()
 OBJ_NOT_FOUND_ERR = 'obj_not_found'
@@ -104,69 +107,71 @@ class XtremIOClient(object):
         elif ver == 'v2':
             return 'https://%s/api/json/v2/types' % self.configuration.san_ip
 
-    @utils.retry(exception.XtremIOArrayBusy,
-                 CONF.xtremio_array_busy_retry_count,
-                 CONF.xtremio_array_busy_retry_interval, 1)
     def req(self, object_type='volumes', method='GET', data=None,
             name=None, idx=None, ver='v1'):
-        if not data:
-            data = {}
-        if name and idx:
-            msg = _("can't handle both name and index in req")
-            LOG.error(msg)
-            raise exception.VolumeDriverException(message=msg)
+        @utils.retry(exception.XtremIOArrayBusy,
+                     self.configuration.xtremio_array_busy_retry_count,
+                     self.configuration.xtremio_array_busy_retry_interval, 1)
+        def _do_req(object_type, method, data, name, idx, ver):
+            if not data:
+                data = {}
+            if name and idx:
+                msg = _("can't handle both name and index in req")
+                LOG.error(msg)
+                raise exception.VolumeDriverException(message=msg)
 
-        url = '%s/%s' % (self.get_base_url(ver), object_type)
-        params = {}
-        key = None
-        if name:
-            params['name'] = name
-            key = name
-        elif idx:
-            url = '%s/%d' % (url, idx)
-            key = str(idx)
-        if method in ('GET', 'DELETE'):
-            params.update(data)
-            self.update_url(params, self.cluster_id)
-        if method != 'GET':
-            self.update_data(data, self.cluster_id)
-            LOG.debug('data: %s', data)
-        LOG.debug('%(type)s %(url)s', {'type': method, 'url': url})
-        try:
-            response = requests.request(method, url, params=params,
-                                        data=json.dumps(data),
-                                        verify=self.verify,
-                                        auth=(self.configuration.san_login,
+            url = '%s/%s' % (self.get_base_url(ver), object_type)
+            params = {}
+            key = None
+            if name:
+                params['name'] = name
+                key = name
+            elif idx:
+                url = '%s/%d' % (url, idx)
+                key = str(idx)
+            if method in ('GET', 'DELETE'):
+                params.update(data)
+                self.update_url(params, self.cluster_id)
+            if method != 'GET':
+                self.update_data(data, self.cluster_id)
+                LOG.debug('data: %s', data)
+            LOG.debug('%(type)s %(url)s', {'type': method, 'url': url})
+            try:
+                response = requests.request(
+                    method, url, params=params, data=json.dumps(data),
+                    verify=self.verify, auth=(self.configuration.san_login,
                                               self.configuration.san_password))
-        except requests.exceptions.RequestException as exc:
-            msg = (_('Exception: %s') % six.text_type(exc))
-            raise exception.VolumeDriverException(message=msg)
+            except requests.exceptions.RequestException as exc:
+                msg = (_('Exception: %s') % six.text_type(exc))
+                raise exception.VolumeDriverException(message=msg)
 
-        if 200 <= response.status_code < 300:
-            if method in ('GET', 'POST'):
-                return response.json()
-            else:
-                return ''
+            if (http_client.OK <= response.status_code <
+                    http_client.MULTIPLE_CHOICES):
+                if method in ('GET', 'POST'):
+                    return response.json()
+                else:
+                    return ''
 
-        self.handle_errors(response, key, object_type)
+            self.handle_errors(response, key, object_type)
+        return _do_req(object_type, method, data, name, idx, ver)
 
     def handle_errors(self, response, key, object_type):
-        if response.status_code == 400:
+        if response.status_code == http_client.BAD_REQUEST:
             error = response.json()
             err_msg = error.get('message')
             if err_msg.endswith(OBJ_NOT_FOUND_ERR):
-                LOG.warning(_LW("object %(key)s of "
-                                "type %(typ)s not found, %(err_msg)s"),
+                LOG.warning("object %(key)s of "
+                            "type %(typ)s not found, %(err_msg)s",
                             {'key': key, 'typ': object_type,
                              'err_msg': err_msg, })
                 raise exception.NotFound()
             elif err_msg == VOL_NOT_UNIQUE_ERR:
-                LOG.error(_LE("can't create 2 volumes with the same name, %s"),
+                LOG.error("can't create 2 volumes with the same name, %s",
                           err_msg)
-                msg = (_('Volume by this name already exists'))
+                msg = _('Volume by this name already exists')
                 raise exception.VolumeBackendAPIException(data=msg)
             elif err_msg == VOL_OBJ_NOT_FOUND_ERR:
-                LOG.error(_LE("Can't find volume to map %(key)s, %(msg)s"),
+                LOG.error("Can't find volume to map %(key)s, %(msg)s",
                           {'key': key, 'msg': err_msg, })
                 raise exception.VolumeNotFound(volume_id=key)
             elif ALREADY_MAPPED_ERR in err_msg:
@@ -352,8 +357,7 @@ class XtremIOClient4(XtremIOClient):
             self.req(typ, 'PUT', data, idx=int(idx))
         except exception.VolumeBackendAPIException:
             # reverting
-            msg = _LE('Failed to rename the created snapshot, reverting.')
-            LOG.error(msg)
+            LOG.error('Failed to rename the created snapshot, reverting.')
             self.req(typ, 'DELETE', idx=int(idx))
             raise
 
@@ -436,14 +440,14 @@ class XtremIOVolumeDriver(san.SanDriver):
             LOG.error(msg)
             raise exception.VolumeBackendAPIException(data=msg)
         else:
-            LOG.info(_LI('XtremIO Cluster version %s'), version_text)
+            LOG.info('XtremIO Cluster version %s', version_text)
         client_ver = '3'
         if ver[0] >= 4:
             # get XMS version
             xms = self.client.req('xms', idx=1)['content']
             xms_version = tuple([int(i) for i in
                                  xms['sw-version'].split('-')[0].split('.')])
-            LOG.info(_LI('XtremIO XMS version %s'), xms_version)
+            LOG.info('XtremIO XMS version %s', version_text)
             if xms_version >= (4, 2):
                 self.client = XtremIOClient42(self.configuration,
                                               self.cluster_id)
@@ -452,7 +456,7 @@ class XtremIOVolumeDriver(san.SanDriver):
                 self.client = XtremIOClient4(self.configuration,
                                              self.cluster_id)
                 client_ver = '4'
-        LOG.info(_LI('Using XtremIO Client %s'), client_ver)
+        LOG.info('Using XtremIO Client %s', client_ver)
 
     def create_volume(self, volume):
         """Creates a volume."""
@@ -512,8 +516,8 @@ class XtremIOVolumeDriver(san.SanDriver):
             try:
                 self.extend_volume(volume, volume['size'])
             except Exception:
-                LOG.error(_LE('failes to extend volume %s, '
-                              'reverting clone operation'), volume['id'])
+                LOG.error('failes to extend volume %s, '
+                          'reverting clone operation', volume['id'])
                 # remove the volume in case resize failed
                 self.delete_volume(volume)
                 raise
@@ -527,7 +531,7 @@ class XtremIOVolumeDriver(san.SanDriver):
         try:
             self.client.req('volumes', 'DELETE', name=volume.name_id)
         except exception.NotFound:
-            LOG.info(_LI("volume %s doesn't exist"), volume.name_id)
+            LOG.info("volume %s doesn't exist", volume.name_id)
 
     def create_snapshot(self, snapshot):
         """Creates a snapshot."""
@@ -538,7 +542,7 @@ class XtremIOVolumeDriver(san.SanDriver):
         try:
             self.client.req('volumes', 'DELETE', name=snapshot.id)
         except exception.NotFound:
-            LOG.info(_LI("snapshot %s doesn't exist"), snapshot.id)
+            LOG.info("snapshot %s doesn't exist", snapshot.id)
 
     def update_migrated_volume(self, ctxt, volume, new_volume,
                                original_volume_status):
@@ -551,8 +555,8 @@ class XtremIOVolumeDriver(san.SanDriver):
             data = {'name': original_name}
             self.client.req('volumes', 'PUT', data, name=current_name)
         except exception.VolumeBackendAPIException:
-            LOG.error(_LE('Unable to rename the logical volume '
-                          'for volume: %s'), original_name)
+            LOG.error('Unable to rename the logical volume '
+                      'for volume: %s', original_name)
             # If the rename fails, _name_id should be set to the new
             # volume id and provider_location should be set to the
             # one from the new volume as well.
@@ -581,7 +585,7 @@ class XtremIOVolumeDriver(san.SanDriver):
                        'reserved_percentage':
                        self.configuration.reserved_percentage,
                        'QoS_support': False,
-                       'multiattach': True,
+                       'multiattach': False,
                        }
         self._stats.update(self.client.get_extra_capabilities())
 
@@ -649,8 +653,8 @@ class XtremIOVolumeDriver(san.SanDriver):
             self.client.req('volumes', 'PUT', name=volume['id'],
                             data={'vol-name': volume['name'] + '-unmanged'})
         except exception.NotFound:
-            LOG.info(_LI("%(typ)s with the name %(name)s wasn't found, "
-                         "can't unmanage") %
+            LOG.info("%(typ)s with the name %(name)s wasn't found, "
+                     "can't unmanage",
                      {'typ': 'Snapshot' if is_snapshot else 'Volume',
                       'name': volume['id']})
             raise exception.VolumeNotFound(volume_id=volume['id'])
@@ -680,10 +684,21 @@ class XtremIOVolumeDriver(san.SanDriver):
     def terminate_connection(self, volume, connector, **kwargs):
         """Disallow connection from connector"""
         tg_index = '1'
-        vol = self.client.req('volumes', name=volume['id'],
-                              data={'prop': 'index'})['content']
 
-        for ig_idx in self._get_ig_indexes_from_initiators(connector):
+        if not connector:
+            vol = self.client.req('volumes', name=volume.id)['content']
+            # foce detach, unmap all IGs from volume
+            IG_OID = 0
+            ig_indexes = [lun_map[IG_OID][XTREMIO_OID_INDEX] for
+                          lun_map in vol['lun-mapping-list']]
+            LOG.info('Force detach volume %(vol)s from luns %(luns)s.',
+                     {'vol': vol['name'], 'luns': ig_indexes})
+        else:
+            vol = self.client.req('volumes', name=volume.id,
+                                  data={'prop': 'index'})['content']
+            ig_indexes = self._get_ig_indexes_from_initiators(connector)
+
+        for ig_idx in ig_indexes:
             lm_name = '%s_%s_%s' % (six.text_type(vol['index']),
                                     six.text_type(ig_idx),
                                     tg_index)
@@ -691,7 +706,7 @@ class XtremIOVolumeDriver(san.SanDriver):
             try:
                 self.client.req('lun-maps', 'DELETE', name=lm_name)
             except exception.NotFound:
-                LOG.warning(_LW("terminate_connection: lun map not found"))
+                LOG.warning("terminate_connection: lun map not found")
 
     def _get_password(self):
         return ''.join(RANDOM.choice
@@ -706,9 +721,9 @@ class XtremIOVolumeDriver(san.SanDriver):
             res = self.client.req('lun-maps', 'POST', data)
 
             lunmap = self._obj_from_result(res)
-            LOG.info(_LI('Created lun-map:\n%s'), lunmap)
+            LOG.info('Created lun-map:\n%s', lunmap)
         except exception.XtremIOAlreadyMappedError:
-            LOG.info(_LI('Volume already mapped, retrieving %(ig)s, %(vol)s'),
+            LOG.info('Volume already mapped, retrieving %(ig)s, %(vol)s',
                      {'ig': ig, 'vol': volume['id']})
             lunmap = self.client.find_lunmap(ig, volume['id'])
         return lunmap
@@ -1038,8 +1053,7 @@ class XtremIOISCSIDriver(XtremIOVolumeDriver, driver.ISCSIDriver):
                                                         discovery_chap)
         # if CHAP was enabled after the initiator was created
         if login_chap and not login_passwd:
-            LOG.info(_LI('initiator has no password while using chap,'
-                         'adding it'))
+            LOG.info('Initiator has no password while using chap, adding it.')
             data = {}
             (login_passwd,
              d_passwd) = self._add_auth(data, login_chap, discovery_chap and
@@ -1061,7 +1075,8 @@ class XtremIOISCSIDriver(XtremIOVolumeDriver, driver.ISCSIDriver):
             properties['discovery_auth_method'] = 'CHAP'
             properties['discovery_auth_username'] = 'chap_user'
             properties['discovery_auth_password'] = discovery_passwd
-        LOG.debug('init conn params:\n%s', properties)
+        LOG.debug('init conn params:\n%s',
+                  strutils.mask_dict_password(properties))
         return {
             'driver_volume_type': 'iscsi',
             'data': properties
@@ -1190,9 +1205,11 @@ class XtremIOFCDriver(XtremIOVolumeDriver,
     def terminate_connection(self, volume, connector, **kwargs):
         (super(XtremIOFCDriver, self)
          .terminate_connection(volume, connector, **kwargs))
-        num_vols = (self.client
-                    .num_of_mapped_volumes(self._get_ig_name(connector)))
-        if num_vols > 0:
+        has_volumes = (not connector
+                       or self.client.
+                       num_of_mapped_volumes(self._get_ig_name(connector)) > 0)
+
+        if has_volumes:
             data = {}
         else:
             i_t_map = {}
