@@ -21,12 +21,15 @@ import threading
 
 from oslo_config import cfg
 from oslo_log import log as logging
+from oslo_log import versionutils
 from oslo_utils import excutils
 import six
 
 from cinder import exception
-from cinder.i18n import _LI, _LW
+from cinder.i18n import _
+from cinder import interface
 from cinder import utils
+from cinder.volume import configuration
 import cinder.volume.driver
 from cinder.volume.drivers.hitachi import hbsd_basiclib as basic_lib
 from cinder.volume.drivers.hitachi import hbsd_common as common
@@ -41,12 +44,18 @@ volume_opts = [
 ]
 
 CONF = cfg.CONF
-CONF.register_opts(volume_opts)
+CONF.register_opts(volume_opts, group=configuration.SHARED_CONF_GROUP)
 
 
+@interface.volumedriver
 class HBSDFCDriver(cinder.volume.driver.FibreChannelDriver):
     VERSION = common.VERSION
 
+    # ThirdPartySystems wiki page
+    CI_WIKI_NAME = ["Hitachi_HBSD_CI", "Hitachi_HBSD2_CI"]
+
+    SUPPORTED = False
+
     def __init__(self, *args, **kwargs):
         os.environ['LANG'] = 'C'
         super(HBSDFCDriver, self).__init__(*args, **kwargs)
@@ -83,7 +92,7 @@ class HBSDFCDriver(cinder.volume.driver.FibreChannelDriver):
             for opt in volume_opts:
                 if not opt.secret:
                     value = getattr(self.configuration, opt.name)
-                    LOG.info(_LI('\t%(name)-35s : %(value)s'),
+                    LOG.info('\t%(name)-35s : %(value)s',
                              {'name': opt.name, 'value': value})
             self.common.command.output_param_to_log(self.configuration)
 
@@ -178,7 +187,7 @@ class HBSDFCDriver(cinder.volume.driver.FibreChannelDriver):
                     try:
                         self._fill_group(hgs, port, host_grp_name, wwns_copy)
                     except Exception as ex:
-                        LOG.warning(_LW('Failed to add host group: %s'), ex)
+                        LOG.warning('Failed to add host group: %s', ex)
                         LOG.warning(basic_lib.set_msg(
                             308, port=port, name=host_grp_name))
 
@@ -305,6 +314,9 @@ class HBSDFCDriver(cinder.volume.driver.FibreChannelDriver):
         self.context = context
         self.common = common.HBSDCommon(self.configuration, self,
                                         context, self.db)
+        msg = _("The HBSD FC driver is deprecated and "
+                "will be removed in P release.")
+        versionutils.report_deprecated_feature(LOG, msg)
 
         self.check_param()
 
@@ -400,7 +412,7 @@ class HBSDFCDriver(cinder.volume.driver.FibreChannelDriver):
 
         return hostgroups
 
-    @fczm_utils.AddFCZone
+    @fczm_utils.add_fc_zone
     def initialize_connection(self, volume, connector):
         self.do_setup_status.wait()
         ldev = self.common.get_ldev(volume)
@@ -428,7 +440,7 @@ class HBSDFCDriver(cinder.volume.driver.FibreChannelDriver):
         self._delete_lun(hostgroups, ldev)
         LOG.debug("*** _terminate_ ***")
 
-    @fczm_utils.RemoveFCZone
+    @fczm_utils.remove_fc_zone
     def terminate_connection(self, volume, connector, **kwargs):
         self.do_setup_status.wait()
         ldev = self.common.get_ldev(volume)
@@ -515,12 +527,6 @@ class HBSDFCDriver(cinder.volume.driver.FibreChannelDriver):
         """
         self.discard_zero_page(dest_vol)
 
-    def restore_backup(self, context, backup, volume, backup_service):
-        self.do_setup_status.wait()
-        super(HBSDFCDriver, self).restore_backup(context, backup,
-                                                 volume, backup_service)
-        self.discard_zero_page(volume)
-
     def manage_existing(self, volume, existing_ref):
         return self.common.manage_existing(volume, existing_ref)
 
