@@ -32,9 +32,6 @@ from oslo_utils import units
 import six
 
 from cinder import exception
-from cinder.i18n import _LE
-from cinder.i18n import _LI
-from cinder.i18n import _LW
 from cinder import utils as cinder_utils
 
 
@@ -75,410 +72,410 @@ class VSPMsg(enum.Enum):
     METHOD_START = {
         'msg_id': 0,
         'loglevel': base_logging.INFO,
-        'msg': _LI('%(method)s starts. (config_group: %(config_group)s)'),
+        'msg': '%(method)s starts. (config_group: %(config_group)s)',
         'suffix': INFO_SUFFIX
     }
     OUTPUT_PARAMETER_VALUES = {
         'msg_id': 1,
         'loglevel': base_logging.INFO,
-        'msg': _LI('The parameter of the storage backend. (config_group: '
-                   '%(config_group)s)'),
+        'msg': 'The parameter of the storage backend. (config_group: '
+               '%(config_group)s)',
         'suffix': INFO_SUFFIX
     }
     METHOD_END = {
         'msg_id': 2,
         'loglevel': base_logging.INFO,
-        'msg': _LI('%(method)s ended. (config_group: %(config_group)s)'),
+        'msg': '%(method)s ended. (config_group: %(config_group)s)',
         'suffix': INFO_SUFFIX
     }
     DRIVER_READY_FOR_USE = {
         'msg_id': 3,
         'loglevel': base_logging.INFO,
-        'msg': _LI('The storage backend can be used. (config_group: '
-                   '%(config_group)s)'),
+        'msg': 'The storage backend can be used. (config_group: '
+               '%(config_group)s)',
         'suffix': INFO_SUFFIX
     }
     DRIVER_INITIALIZATION_START = {
         'msg_id': 4,
         'loglevel': base_logging.INFO,
-        'msg': _LI('Initialization of %(driver)s %(version)s started.'),
+        'msg': 'Initialization of %(driver)s %(version)s started.',
         'suffix': INFO_SUFFIX
     }
     SET_CONFIG_VALUE = {
         'msg_id': 5,
         'loglevel': base_logging.INFO,
-        'msg': _LI('Set %(object)s to %(value)s.'),
+        'msg': 'Set %(object)s to %(value)s.',
         'suffix': INFO_SUFFIX
     }
     OBJECT_CREATED = {
         'msg_id': 6,
         'loglevel': base_logging.INFO,
-        'msg': _LI('Created %(object)s. (%(details)s)'),
+        'msg': 'Created %(object)s. (%(details)s)',
         'suffix': INFO_SUFFIX
     }
     INVALID_LDEV_FOR_UNMAPPING = {
         'msg_id': 302,
         'loglevel': base_logging.WARNING,
-        'msg': _LW('Failed to specify a logical device for the volume '
-                   '%(volume_id)s to be unmapped.'),
+        'msg': 'Failed to specify a logical device for the volume '
+               '%(volume_id)s to be unmapped.',
         'suffix': WARNING_SUFFIX
     }
     INVALID_LDEV_FOR_DELETION = {
         'msg_id': 304,
         'loglevel': base_logging.WARNING,
-        'msg': _LW('Failed to specify a logical device to be deleted. '
-                   '(method: %(method)s, id: %(id)s)'),
+        'msg': 'Failed to specify a logical device to be deleted. '
+               '(method: %(method)s, id: %(id)s)',
         'suffix': WARNING_SUFFIX
     }
     DELETE_TARGET_FAILED = {
         'msg_id': 306,
         'loglevel': base_logging.WARNING,
-        'msg': _LW('A host group or an iSCSI target could not be deleted. '
-                   '(port: %(port)s, gid: %(id)s)'),
+        'msg': 'A host group or an iSCSI target could not be deleted. '
+               '(port: %(port)s, gid: %(id)s)',
         'suffix': WARNING_SUFFIX
     }
     CREATE_HOST_GROUP_FAILED = {
         'msg_id': 308,
         'loglevel': base_logging.WARNING,
-        'msg': _LW('A host group could not be added. (port: %(port)s)'),
+        'msg': 'A host group could not be added. (port: %(port)s)',
         'suffix': WARNING_SUFFIX
     }
     CREATE_ISCSI_TARGET_FAILED = {
         'msg_id': 309,
         'loglevel': base_logging.WARNING,
-        'msg': _LW('An iSCSI target could not be added. (port: %(port)s)'),
+        'msg': 'An iSCSI target could not be added. (port: %(port)s)',
         'suffix': WARNING_SUFFIX
     }
     UNMAP_LDEV_FAILED = {
         'msg_id': 310,
         'loglevel': base_logging.WARNING,
-        'msg': _LW('Failed to unmap a logical device. (LDEV: %(ldev)s)'),
+        'msg': 'Failed to unmap a logical device. (LDEV: %(ldev)s)',
         'suffix': WARNING_SUFFIX
     }
     DELETE_LDEV_FAILED = {
         'msg_id': 313,
         'loglevel': base_logging.WARNING,
-        'msg': _LW('Failed to delete a logical device. (LDEV: %(ldev)s)'),
+        'msg': 'Failed to delete a logical device. (LDEV: %(ldev)s)',
         'suffix': WARNING_SUFFIX
     }
     MAP_LDEV_FAILED = {
         'msg_id': 314,
         'loglevel': base_logging.WARNING,
-        'msg': _LW('Failed to map a logical device. (LDEV: %(ldev)s, port: '
-                   '%(port)s, id: %(id)s, lun: %(lun)s)'),
+        'msg': 'Failed to map a logical device. (LDEV: %(ldev)s, port: '
+               '%(port)s, id: %(id)s, lun: %(lun)s)',
         'suffix': WARNING_SUFFIX
     }
     DISCARD_ZERO_PAGE_FAILED = {
         'msg_id': 315,
         'loglevel': base_logging.WARNING,
-        'msg': _LW('Failed to perform a zero-page reclamation. (LDEV: '
-                   '%(ldev)s)'),
+        'msg': 'Failed to perform a zero-page reclamation. (LDEV: '
+               '%(ldev)s)',
         'suffix': WARNING_SUFFIX
     }
     ADD_HBA_WWN_FAILED = {
         'msg_id': 317,
         'loglevel': base_logging.WARNING,
-        'msg': _LW('Failed to assign the WWN. (port: %(port)s, gid: %(gid)s, '
-                   'wwn: %(wwn)s)'),
+        'msg': 'Failed to assign the WWN. (port: %(port)s, gid: %(gid)s, '
+               'wwn: %(wwn)s)',
         'suffix': WARNING_SUFFIX
     }
     LDEV_NOT_EXIST = {
         'msg_id': 319,
         'loglevel': base_logging.WARNING,
-        'msg': _LW('The logical device does not exist in the storage system. '
-                   '(LDEV: %(ldev)s)'),
+        'msg': 'The logical device does not exist in the storage system. '
+               '(LDEV: %(ldev)s)',
         'suffix': WARNING_SUFFIX
     }
     HORCM_START_FAILED = {
         'msg_id': 320,
         'loglevel': base_logging.WARNING,
-        'msg': _LW('Failed to start HORCM. (inst: %(inst)s)'),
+        'msg': 'Failed to start HORCM. (inst: %(inst)s)',
         'suffix': WARNING_SUFFIX
     }
     HORCM_RESTART_FOR_SI_FAILED = {
         'msg_id': 322,
         'loglevel': base_logging.WARNING,
-        'msg': _LW('Failed to reload the configuration of full copy pair. '
-                   '(inst: %(inst)s)'),
+        'msg': 'Failed to reload the configuration of full copy pair. '
+               '(inst: %(inst)s)',
         'suffix': WARNING_SUFFIX
     }
     HORCM_LOGIN_FAILED = {
         'msg_id': 323,
         'loglevel': base_logging.WARNING,
-        'msg': _LW('Failed to perform user authentication of HORCM. '
-                   '(user: %(user)s)'),
+        'msg': 'Failed to perform user authentication of HORCM. '
+               '(user: %(user)s)',
         'suffix': WARNING_SUFFIX
     }
     DELETE_SI_PAIR_FAILED = {
         'msg_id': 324,
         'loglevel': base_logging.WARNING,
-        'msg': _LW('Failed to delete full copy pair. (P-VOL: %(pvol)s, S-VOL: '
-                   '%(svol)s)'),
+        'msg': 'Failed to delete full copy pair. (P-VOL: %(pvol)s, S-VOL: '
+               '%(svol)s)',
         'suffix': WARNING_SUFFIX
     }
     DELETE_TI_PAIR_FAILED = {
         'msg_id': 325,
         'loglevel': base_logging.WARNING,
-        'msg': _LW('Failed to delete thin copy pair. (P-VOL: %(pvol)s, S-VOL: '
-                   '%(svol)s)'),
+        'msg': 'Failed to delete thin copy pair. (P-VOL: %(pvol)s, S-VOL: '
+               '%(svol)s)',
         'suffix': WARNING_SUFFIX
     }
     WAIT_SI_PAIR_STATUS_FAILED = {
         'msg_id': 326,
         'loglevel': base_logging.WARNING,
-        'msg': _LW('Failed to change the status of full copy pair. (P-VOL: '
-                   '%(pvol)s, S-VOL: %(svol)s)'),
+        'msg': 'Failed to change the status of full copy pair. (P-VOL: '
+               '%(pvol)s, S-VOL: %(svol)s)',
         'suffix': WARNING_SUFFIX
     }
     DELETE_DEVICE_GRP_FAILED = {
         'msg_id': 327,
         'loglevel': base_logging.WARNING,
-        'msg': _LW('Failed to delete the configuration of full copy pair. '
-                   '(P-VOL: %(pvol)s, S-VOL: %(svol)s)'),
+        'msg': 'Failed to delete the configuration of full copy pair. '
+               '(P-VOL: %(pvol)s, S-VOL: %(svol)s)',
         'suffix': WARNING_SUFFIX
     }
     DISCONNECT_VOLUME_FAILED = {
         'msg_id': 329,
         'loglevel': base_logging.WARNING,
-        'msg': _LW('Failed to detach the logical device. (LDEV: %(ldev)s, '
-                   'reason: %(reason)s)'),
+        'msg': 'Failed to detach the logical device. (LDEV: %(ldev)s, '
+               'reason: %(reason)s)',
         'suffix': WARNING_SUFFIX
     }
     STORAGE_COMMAND_FAILED = {
         'msg_id': 600,
         'loglevel': base_logging.ERROR,
-        'msg': _LE('The command %(cmd)s failed. (ret: %(ret)s, stdout: '
-                   '%(out)s, stderr: %(err)s)'),
+        'msg': 'The command %(cmd)s failed. (ret: %(ret)s, stdout: '
+               '%(out)s, stderr: %(err)s)',
         'suffix': ERROR_SUFFIX
     }
     INVALID_PARAMETER = {
         'msg_id': 601,
         'loglevel': base_logging.ERROR,
-        'msg': _LE('A parameter is invalid. (%(param)s)'),
+        'msg': 'A parameter is invalid. (%(param)s)',
         'suffix': ERROR_SUFFIX
     }
     INVALID_PARAMETER_VALUE = {
         'msg_id': 602,
         'loglevel': base_logging.ERROR,
-        'msg': _LE('A parameter value is invalid. (%(meta)s)'),
+        'msg': 'A parameter value is invalid. (%(meta)s)',
         'suffix': ERROR_SUFFIX
     }
     HORCM_SHUTDOWN_FAILED = {
         'msg_id': 608,
         'loglevel': base_logging.ERROR,
-        'msg': _LE('Failed to shutdown HORCM. (inst: %(inst)s)'),
+        'msg': 'Failed to shutdown HORCM. (inst: %(inst)s)',
         'suffix': ERROR_SUFFIX
     }
     HORCM_RESTART_FAILED = {
         'msg_id': 609,
         'loglevel': base_logging.ERROR,
-        'msg': _LE('Failed to restart HORCM. (inst: %(inst)s)'),
+        'msg': 'Failed to restart HORCM. (inst: %(inst)s)',
         'suffix': ERROR_SUFFIX
     }
     SI_PAIR_STATUS_WAIT_TIMEOUT = {
         'msg_id': 610,
         'loglevel': base_logging.ERROR,
-        'msg': _LE('The status change of full copy pair could not be '
-                   'completed. (S-VOL: %(svol)s)'),
+        'msg': 'The status change of full copy pair could not be '
+               'completed. (S-VOL: %(svol)s)',
         'suffix': ERROR_SUFFIX
     }
     TI_PAIR_STATUS_WAIT_TIMEOUT = {
         'msg_id': 611,
         'loglevel': base_logging.ERROR,
-        'msg': _LE('The status change of thin copy pair could not be '
-                   'completed. (S-VOL: %(svol)s)'),
+        'msg': 'The status change of thin copy pair could not be '
+               'completed. (S-VOL: %(svol)s)',
         'suffix': ERROR_SUFFIX
     }
     INVALID_LDEV_STATUS_FOR_COPY = {
         'msg_id': 612,
         'loglevel': base_logging.ERROR,
-        'msg': _LE('The source logical device to be replicated does not exist '
-                   'in the storage system. (LDEV: %(ldev)s)'),
+        'msg': 'The source logical device to be replicated does not exist '
+               'in the storage system. (LDEV: %(ldev)s)',
         'suffix': ERROR_SUFFIX
     }
     INVALID_LDEV_FOR_EXTENSION = {
         'msg_id': 613,
         'loglevel': base_logging.ERROR,
-        'msg': _LE('The volume %(volume_id)s to be extended was not found.'),
+        'msg': 'The volume %(volume_id)s to be extended was not found.',
         'suffix': ERROR_SUFFIX
     }
     NO_HBA_WWN_ADDED_TO_HOST_GRP = {
         'msg_id': 614,
         'loglevel': base_logging.ERROR,
-        'msg': _LE('No WWN is assigned. (port: %(port)s, gid: %(gid)s)'),
+        'msg': 'No WWN is assigned. (port: %(port)s, gid: %(gid)s)',
         'suffix': ERROR_SUFFIX
     }
     NO_AVAILABLE_MIRROR_UNIT = {
         'msg_id': 615,
         'loglevel': base_logging.ERROR,
-        'msg': _LE('A pair could not be created. The maximum number of pair '
-                   'is exceeded. (copy method: %(copy_method)s, P-VOL: '
-                   '%(pvol)s)'),
+        'msg': 'A pair could not be created. The maximum number of pair '
+               'is exceeded. (copy method: %(copy_method)s, P-VOL: '
+               '%(pvol)s)',
         'suffix': ERROR_SUFFIX
     }
     UNABLE_TO_DELETE_PAIR = {
         'msg_id': 616,
         'loglevel': base_logging.ERROR,
-        'msg': _LE('A pair cannot be deleted. (P-VOL: %(pvol)s, S-VOL: '
-                   '%(svol)s)'),
+        'msg': 'A pair cannot be deleted. (P-VOL: %(pvol)s, S-VOL: '
+               '%(svol)s)',
         'suffix': ERROR_SUFFIX
     }
     INVALID_VOLUME_SIZE_FOR_COPY = {
         'msg_id': 617,
         'loglevel': base_logging.ERROR,
-        'msg': _LE('Failed to create a volume from a %(type)s. The size of '
-                   'the new volume must be equal to or greater than the size '
-                   'of the original %(type)s. (new volume: %(volume_id)s)'),
+        'msg': 'Failed to create a volume from a %(type)s. The size of '
+               'the new volume must be equal to or greater than the size '
+               'of the original %(type)s. (new volume: %(volume_id)s)',
         'suffix': ERROR_SUFFIX
     }
     INVALID_VOLUME_TYPE_FOR_EXTEND = {
         'msg_id': 618,
         'loglevel': base_logging.ERROR,
-        'msg': _LE('The volume %(volume_id)s could not be extended. The '
-                   'volume type must be Normal.'),
+        'msg': 'The volume %(volume_id)s could not be extended. The '
+               'volume type must be Normal.',
         'suffix': ERROR_SUFFIX
     }
     INVALID_LDEV_FOR_CONNECTION = {
         'msg_id': 619,
         'loglevel': base_logging.ERROR,
-        'msg': _LE('The volume %(volume_id)s to be mapped was not found.'),
+        'msg': 'The volume %(volume_id)s to be mapped was not found.',
         'suffix': ERROR_SUFFIX
     }
     POOL_INFO_RETRIEVAL_FAILED = {
         'msg_id': 620,
         'loglevel': base_logging.ERROR,
-        'msg': _LE('Failed to provide information about a pool. (pool: '
-                   '%(pool)s)'),
+        'msg': 'Failed to provide information about a pool. (pool: '
+               '%(pool)s)',
         'suffix': ERROR_SUFFIX
     }
     INVALID_VOLUME_SIZE_FOR_TI = {
         'msg_id': 621,
         'loglevel': base_logging.ERROR,
-        'msg': _LE('Failed to create a volume from a %(type)s. The size of '
-                   'the new volume must be equal to the size of the original '
-                   '%(type)s when the new volume is created by '
-                   '%(copy_method)s. (new volume: %(volume_id)s)'),
+        'msg': 'Failed to create a volume from a %(type)s. The size of '
+               'the new volume must be equal to the size of the original '
+               '%(type)s when the new volume is created by '
+               '%(copy_method)s. (new volume: %(volume_id)s)',
         'suffix': ERROR_SUFFIX
     }
     INVALID_LDEV_FOR_VOLUME_COPY = {
         'msg_id': 624,
         'loglevel': base_logging.ERROR,
-        'msg': _LE('The %(type)s %(id)s source to be replicated was not '
-                   'found.'),
+        'msg': 'The %(type)s %(id)s source to be replicated was not '
+               'found.',
         'suffix': ERROR_SUFFIX
     }
     CREATE_HORCM_CONF_FILE_FAILED = {
         'msg_id': 632,
         'loglevel': base_logging.ERROR,
-        'msg': _LE('Failed to open a file. (file: %(file)s, ret: %(ret)s, '
-                   'stderr: %(err)s)'),
+        'msg': 'Failed to open a file. (file: %(file)s, ret: %(ret)s, '
+               'stderr: %(err)s)',
         'suffix': ERROR_SUFFIX
     }
     CONNECT_VOLUME_FAILED = {
         'msg_id': 634,
         'loglevel': base_logging.ERROR,
-        'msg': _LE('Failed to attach the logical device. (LDEV: %(ldev)s, '
-                   'reason: %(reason)s)'),
+        'msg': 'Failed to attach the logical device. (LDEV: %(ldev)s, '
+               'reason: %(reason)s)',
         'suffix': ERROR_SUFFIX
     }
     CREATE_LDEV_FAILED = {
         'msg_id': 636,
         'loglevel': base_logging.ERROR,
-        'msg': _LE('Failed to add the logical device.'),
+        'msg': 'Failed to add the logical device.',
         'suffix': ERROR_SUFFIX
     }
     ADD_PAIR_TARGET_FAILED = {
         'msg_id': 638,
         'loglevel': base_logging.ERROR,
-        'msg': _LE('Failed to add the pair target.'),
+        'msg': 'Failed to add the pair target.',
         'suffix': ERROR_SUFFIX
     }
     NO_MAPPING_FOR_LDEV = {
         'msg_id': 639,
         'loglevel': base_logging.ERROR,
-        'msg': _LE('Failed to map a logical device to any pair targets. '
-                   '(LDEV: %(ldev)s)'),
+        'msg': 'Failed to map a logical device to any pair targets. '
+               '(LDEV: %(ldev)s)',
         'suffix': ERROR_SUFFIX
     }
     POOL_NOT_FOUND = {
         'msg_id': 640,
         'loglevel': base_logging.ERROR,
-        'msg': _LE('A pool could not be found. (pool: %(pool)s)'),
+        'msg': 'A pool could not be found. (pool: %(pool)s)',
         'suffix': ERROR_SUFFIX
     }
     NO_AVAILABLE_RESOURCE = {
         'msg_id': 648,
         'loglevel': base_logging.ERROR,
-        'msg': _LE('There are no resources available for use. (resource: '
-                   '%(resource)s)'),
+        'msg': 'There are no resources available for use. (resource: '
+               '%(resource)s)',
         'suffix': ERROR_SUFFIX
     }
     NO_CONNECTED_TARGET = {
         'msg_id': 649,
         'loglevel': base_logging.ERROR,
-        'msg': _LE('The host group or iSCSI target was not found.'),
+        'msg': 'The host group or iSCSI target was not found.',
         'suffix': ERROR_SUFFIX
     }
     RESOURCE_NOT_FOUND = {
         'msg_id': 650,
         'loglevel': base_logging.ERROR,
-        'msg': _LE('The resource %(resource)s was not found.'),
+        'msg': 'The resource %(resource)s was not found.',
         'suffix': ERROR_SUFFIX
     }
     LDEV_DELETION_WAIT_TIMEOUT = {
         'msg_id': 652,
         'loglevel': base_logging.ERROR,
-        'msg': _LE('Failed to delete a logical device. (LDEV: %(ldev)s)'),
+        'msg': 'Failed to delete a logical device. (LDEV: %(ldev)s)',
         'suffix': ERROR_SUFFIX
     }
     LDEV_CREATION_WAIT_TIMEOUT = {
         'msg_id': 653,
         'loglevel': base_logging.ERROR,
-        'msg': _LE('The creation of a logical device could not be completed. '
-                   '(LDEV: %(ldev)s)'),
+        'msg': 'The creation of a logical device could not be completed. '
+               '(LDEV: %(ldev)s)',
         'suffix': ERROR_SUFFIX
     }
     INVALID_LDEV_ATTR_FOR_MANAGE = {
         'msg_id': 702,
         'loglevel': base_logging.ERROR,
-        'msg': _LE('Failed to manage the specified LDEV (%(ldev)s). The LDEV '
-                   'must be an unpaired %(ldevtype)s.'),
+        'msg': 'Failed to manage the specified LDEV (%(ldev)s). The LDEV '
+               'must be an unpaired %(ldevtype)s.',
         'suffix': ERROR_SUFFIX
     }
     INVALID_LDEV_SIZE_FOR_MANAGE = {
         'msg_id': 703,
         'loglevel': base_logging.ERROR,
-        'msg': _LE('Failed to manage the specified LDEV (%(ldev)s). The LDEV '
-                   'size must be expressed in gigabytes.'),
+        'msg': 'Failed to manage the specified LDEV (%(ldev)s). The LDEV '
+               'size must be expressed in gigabytes.',
         'suffix': ERROR_SUFFIX
     }
     INVALID_LDEV_PORT_FOR_MANAGE = {
         'msg_id': 704,
         'loglevel': base_logging.ERROR,
-        'msg': _LE('Failed to manage the specified LDEV (%(ldev)s). The LDEV '
-                   'must not be mapped.'),
+        'msg': 'Failed to manage the specified LDEV (%(ldev)s). The LDEV '
+               'must not be mapped.',
         'suffix': ERROR_SUFFIX
     }
     INVALID_LDEV_TYPE_FOR_UNMANAGE = {
         'msg_id': 706,
         'loglevel': base_logging.ERROR,
-        'msg': _LE('Failed to unmanage the volume %(volume_id)s. The volume '
-                   'type must be %(volume_type)s.'),
+        'msg': 'Failed to unmanage the volume %(volume_id)s. The volume '
+               'type must be %(volume_type)s.',
         'suffix': ERROR_SUFFIX
     }
     INVALID_LDEV_FOR_MANAGE = {
         'msg_id': 707,
         'loglevel': base_logging.ERROR,
-        'msg': _LE('No valid value is specified for "source-id". A valid LDEV '
-                   'number must be specified in "source-id" to manage the '
-                   'volume.'),
+        'msg': 'No valid value is specified for "source-id". A valid LDEV '
+               'number must be specified in "source-id" to manage the '
+               'volume.',
         'suffix': ERROR_SUFFIX
     }
     VOLUME_COPY_FAILED = {
         'msg_id': 722,
         'loglevel': base_logging.ERROR,
-        'msg': _LE('Failed to copy a volume. (copy method: %(copy_method)s, '
-                   'P-VOL: %(pvol)s, S-VOL: %(svol)s)'),
+        'msg': 'Failed to copy a volume. (copy method: %(copy_method)s, '
+               'P-VOL: %(pvol)s, S-VOL: %(svol)s)',
         'suffix': ERROR_SUFFIX
     }
 
@@ -615,14 +612,14 @@ def check_opt_value(conf, names):
 
 def output_storage_cli_info(name, version):
     """Output storage CLI info to the log file."""
-    LOG.info(_LI('\t%(name)-35s%(version)s'),
+    LOG.info('\t%(name)-35s%(version)s',
              {'name': name + ' version: ', 'version': version})
 
 
 def output_opt_info(conf, names):
     """Output parameter names and values to the log file."""
     for name in names:
-        LOG.info(_LI('\t%(name)-35s%(attr)s'),
+        LOG.info('\t%(name)-35s%(attr)s',
                  {'name': name + ': ', 'attr': getattr(conf, name)})
 
 
