@@ -111,10 +111,13 @@ class HPE3PARFCDriver(driver.ManageableVD,
         3.0.10 - Added Entry point tracing
         3.0.11 - Handle manage and unmanage hosts present. bug #1648067
         3.0.12 - Adds consistency group capability in generic volume groups.
+        3.0.13 - Create one vlun in single path configuration. bug #1727176
+        3.0.14 - Added check to remove FC zones. bug #1730720
+        3.0.15 - Create FC vlun as host sees. bug #1734505
 
     """
 
-    VERSION = "3.0.12"
+    VERSION = "3.0.15"
 
     # The name of the CI wiki page.
     CI_WIKI_NAME = "HPE_Storage_CI"
@@ -287,67 +290,26 @@ class HPE3PARFCDriver(driver.ManageableVD,
             host = self._create_host(common, volume, connector)
             target_wwns, init_targ_map, numPaths = \
                 self._build_initiator_target_map(common, connector)
+            if not connector.get('multipath'):
+                target_wwns = target_wwns[:1]
+                initiator = connector.get('wwpns')[0]
+                init_targ_map[initiator] = init_targ_map[initiator][:1]
             # check if a VLUN already exists for this host
             existing_vlun = common.find_existing_vlun(volume, host)
 
             vlun = None
             if existing_vlun is None:
                 # now that we have a host, create the VLUN
-                nsp = None
-                lun_id = None
-                active_fc_port_list = common.get_active_fc_target_ports()
-
-                if self.lookup_service:
-                    if not init_targ_map:
-                        msg = _("Setup is incomplete. Device mapping "
-                                "not found from FC network. "
-                                "Cannot perform VLUN creation.")
-                        LOG.error(msg)
-                        raise exception.FCSanLookupServiceException(msg)
-
-                    for target_wwn in target_wwns:
-                        for port in active_fc_port_list:
-                            if port['portWWN'].lower() == target_wwn.lower():
-                                nsp = port['nsp']
-                                vlun = common.create_vlun(volume,
-                                                          host,
-                                                          nsp,
-                                                          lun_id=lun_id)
-                                if lun_id is None:
-                                    lun_id = vlun['lun']
-                                break
-                else:
-                    init_targ_map.clear()
-                    del target_wwns[:]
-                    host_connected_nsp = []
-                    for fcpath in host['FCPaths']:
-                        if 'portPos' in fcpath:
-                            host_connected_nsp.append(
-                                common.build_nsp(fcpath['portPos']))
+                if self.lookup_service is not None and numPaths == 1:
+                    nsp = None
+                    active_fc_port_list = common.get_active_fc_target_ports()
                     for port in active_fc_port_list:
-                        if (
-                            port['type'] == common.client.PORT_TYPE_HOST and
-                            port['nsp'] in host_connected_nsp
-                        ):
+                        if port['portWWN'].lower() == target_wwns[0].lower():
                             nsp = port['nsp']
-                            vlun = common.create_vlun(volume,
-                                                      host,
-                                                      nsp,
-                                                      lun_id=lun_id)
-                            target_wwns.append(port['portWWN'])
-                            if vlun['remoteName'] in init_targ_map:
-                                init_targ_map[vlun['remoteName']].append(
-                                    port['portWWN'])
-                            else:
-                                init_targ_map[vlun['remoteName']] = [
-                                    port['portWWN']]
-                            if lun_id is None:
-                                lun_id = vlun['lun']
-                if lun_id is None:
-                    # New vlun creation failed
-                    msg = _('No new vlun(s) were created')
-                    LOG.error(msg)
-                    raise exception.VolumeDriverException(msg)
+                            break
+                    vlun = common.create_vlun(volume, host, nsp)
+                else:
+                    vlun = common.create_vlun(volume, host)
             else:
                 vlun = existing_vlun
 
@@ -376,10 +338,23 @@ class HPE3PARFCDriver(driver.ManageableVD,
             info = {'driver_volume_type': 'fibre_channel',
                     'data': {}}
 
+            zone_remove = True
+
             try:
-                common.client.getHostVLUNs(hostname)
+                vluns = common.client.getHostVLUNs(hostname)
             except hpeexceptions.HTTPNotFound:
                 # No more exports for this host.
+                pass
+            else:
+                # Vlun exists, so check for wwpn entry.
+                for wwpn in connector.get('wwpns'):
+                    for vlun in vluns:
+                        if vlun.get('active') and \
+                           vlun.get('remoteName') == wwpn.upper():
+                            zone_remove = False
+                            break
+
+            if zone_remove:
                 LOG.info("Need to remove FC Zone, building initiator "
                          "target map")
 
@@ -388,6 +363,7 @@ class HPE3PARFCDriver(driver.ManageableVD,
 
                 info['data'] = {'target_wwn': target_wwns,
                                 'initiator_target_map': init_targ_map}
+
             return info
 
         finally:
@@ -494,6 +470,8 @@ class HPE3PARFCDriver(driver.ManageableVD,
         hostname = common._safe_hostname(connector['host'])
         cpg = common.get_cpg(volume, allowSnap=True)
         domain = common.get_domain(cpg)
+        if not connector.get('multipath'):
+            connector['wwpns'] = connector['wwpns'][:1]
         try:
             host = common._get_3par_host(hostname)
             # Check whether host with wwn of initiator present on 3par
