@@ -471,7 +471,7 @@ class HuaweiBaseDriver(driver.VolumeDriver):
                 smart_qos.remove(qos_id, lun_id)
 
         metadata = huawei_utils.get_lun_metadata(volume)
-        if 'hypermetro_id' in metadata:
+        if metadata.get('hypermetro_id'):
             metro = hypermetro.HuaweiHyperMetro(self.client,
                                                 self.rmt_client,
                                                 self.configuration)
@@ -590,18 +590,21 @@ class HuaweiBaseDriver(driver.VolumeDriver):
         orig_lun_name = huawei_utils.encode_name(volume.id)
         new_lun_id, lun_wwn = huawei_utils.get_volume_lun_id(
             self.client, new_volume)
+        new_metadata = huawei_utils.get_lun_metadata(new_volume)
+        model_update = {
+            'provider_location': huawei_utils.to_string(**new_metadata),
+        }
 
         try:
             self.client.rename_lun(new_lun_id, orig_lun_name)
         except exception.VolumeBackendAPIException:
             LOG.error('Unable to rename lun %s on array.', new_lun_id)
-            return {'_name_id': new_volume.name_id}
-
-        LOG.debug("Renamed lun %(id)s to %(name)s successfully.",
-                  {'id': new_lun_id,
-                   'name': orig_lun_name})
-
-        model_update = {'_name_id': None}
+            model_update['_name_id'] = new_volume.name_id
+        else:
+            LOG.debug("Renamed lun %(id)s to %(name)s successfully.",
+                      {'id': new_lun_id,
+                       'name': orig_lun_name})
+            model_update['_name_id'] = None
 
         return model_update
 
@@ -2176,7 +2179,7 @@ class HuaweiFCDriver(HuaweiBaseDriver, driver.FibreChannelDriver):
 
         metadata = huawei_utils.get_lun_metadata(volume)
         LOG.info("initialize_connection, metadata is: %s.", metadata)
-        hypermetro_lun = 'hypermetro_id' in metadata
+        hypermetro_lun = metadata.get('hypermetro_id') is not None
 
         map_info = self.client.do_mapping(lun_id, hostgroup_id,
                                           host_id, portg_id,
@@ -2334,7 +2337,7 @@ class HuaweiFCDriver(HuaweiBaseDriver, driver.FibreChannelDriver):
         metadata = huawei_utils.get_lun_metadata(volume)
         LOG.info("Detach Volume, metadata is: %s.", metadata)
 
-        if 'hypermetro_id' in metadata:
+        if metadata.get('hypermetro_id'):
             hyperm = hypermetro.HuaweiHyperMetro(self.client,
                                                  self.rmt_client,
                                                  self.configuration)
