@@ -25,6 +25,7 @@ from oslo_utils import importutils
 
 from cinder import exception
 from cinder import interface
+from cinder.volume import configuration
 from cinder.volume import driver
 from cinder.volume.drivers.san import san
 from cinder.zonemanager import utils as fczm_utils
@@ -52,7 +53,7 @@ driver_opts = [
 ]
 
 CONF = cfg.CONF
-CONF.register_opts(driver_opts)
+CONF.register_opts(driver_opts, group=configuration.SHARED_CONF_GROUP)
 
 LOG = logging.getLogger(__name__)
 
@@ -67,9 +68,18 @@ class IBMStorageDriver(san.SanDriver,
     IBM Storage driver is a unified Volume driver for IBM XIV, Spectrum
     Accelerate, FlashSystem A9000, FlashSystem A9000R and DS8000 storage
     systems.
+
+    Version history:
+
+    .. code-block:: none
+
+        2.0 - First open source driver version
+        2.1.0 - Support Consistency groups through Generic volume groups
+              - Support XIV/A9000 Volume independent QoS
+              - Support Consistency groups replication
     """
 
-    VERSION = "2.0.0"
+    VERSION = "2.1.0"
 
     # ThirdPartySystems wiki page
     CI_WIKI_NAME = "IBM_STORAGE_CI"
@@ -207,11 +217,11 @@ class IBMStorageDriver(san.SanDriver,
 
         return self.proxy.thaw_backend(context)
 
-    def failover_host(self, context, volumes, secondary_id=None):
+    def failover_host(self, context, volumes, secondary_id=None, groups=None):
         """Failover a backend to a secondary replication target. """
 
         return self.proxy.failover_host(
-            context, volumes, secondary_id)
+            context, volumes, secondary_id, groups)
 
     def get_replication_status(self, context, volume):
         """Return replication status."""
@@ -223,41 +233,61 @@ class IBMStorageDriver(san.SanDriver,
 
         return self.proxy.retype(ctxt, volume, new_type, diff, host)
 
-    def create_consistencygroup(self, context, group):
-        """Creates a consistency group."""
+    def create_group(self, context, group):
+        """Creates a group."""
 
-        return self.proxy.create_consistencygroup(context, group)
+        return self.proxy.create_group(context, group)
 
-    def delete_consistencygroup(self, context, group, volumes):
-        """Deletes a consistency group."""
+    def delete_group(self, context, group, volumes):
+        """Deletes a group."""
 
-        return self.proxy.delete_consistencygroup(
-            context, group, volumes)
+        return self.proxy.delete_group(context, group, volumes)
 
-    def create_cgsnapshot(self, context, cgsnapshot, snapshots):
-        """Creates a consistency group snapshot."""
+    def create_group_snapshot(self, context, group_snapshot, snapshots):
+        """Creates a group snapshot."""
 
-        return self.proxy.create_cgsnapshot(
-            context, cgsnapshot, snapshots)
+        return self.proxy.create_group_snapshot(
+            context, group_snapshot, snapshots)
 
-    def delete_cgsnapshot(self, context, cgsnapshot, snapshots):
-        """Deletes a consistency group snapshot."""
+    def delete_group_snapshot(self, context, group_snapshot, snapshots):
+        """Deletes a group snapshot."""
 
-        return self.proxy.delete_cgsnapshot(
-            context, cgsnapshot, snapshots)
+        return self.proxy.delete_group_snapshot(
+            context, group_snapshot, snapshots)
 
-    def update_consistencygroup(self, context, group,
-                                add_volumes, remove_volumes):
-        """Adds or removes volume(s) to/from an existing consistency group."""
+    def update_group(self, context, group, add_volumes, remove_volumes):
+        """Adds or removes volume(s) to/from an existing group."""
 
-        return self.proxy.update_consistencygroup(
+        return self.proxy.update_group(
             context, group, add_volumes, remove_volumes)
 
-    def create_consistencygroup_from_src(
-            self, context, group, volumes, cgsnapshot, snapshots,
+    def create_group_from_src(
+            self, context, group, volumes, group_snapshot, snapshots,
             source_cg=None, source_vols=None):
-        """Creates a consistencygroup from source."""
+        """Creates a group from source."""
 
-        return self.proxy.create_consistencygroup_from_src(
-            context, group, volumes, cgsnapshot, snapshots,
+        return self.proxy.create_group_from_src(
+            context, group, volumes, group_snapshot, snapshots,
             source_cg, source_vols)
+
+    def enable_replication(self, context, group, volumes):
+        """Enable replication."""
+
+        return self.proxy.enable_replication(context, group, volumes)
+
+    def disable_replication(self, context, group, volumes):
+        """Disable replication."""
+
+        return self.proxy.disable_replication(context, group, volumes)
+
+    def failover_replication(self, context, group, volumes,
+                             secondary_backend_id):
+        """Failover replication."""
+
+        return self.proxy.failover_replication(context, group, volumes,
+                                               secondary_backend_id)
+
+    def get_replication_error_status(self, context, groups):
+        """Returns error info for replicated groups and its volumes."""
+
+        return self.proxy.get_replication_error_status(context, groups)
