@@ -13,13 +13,11 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 """Volume driver for Kaminario K2 all-flash arrays."""
-import six
-
 from oslo_log import log as logging
 
 from cinder import coordination
 from cinder import exception
-from cinder.i18n import _, _LE
+from cinder.i18n import _
 from cinder.objects import fields
 from cinder.volume.drivers.kaminario import kaminario_common as common
 from cinder.zonemanager import utils as fczm_utils
@@ -138,10 +136,9 @@ class KaminarioFCDriver(common.KaminarioCinderDriver):
                 host = self.client.new("hosts", name=host_name,
                                        type="Linux").save()
             except Exception as ex:
-                LOG.exception(_LE("Unable to create host : %s in K2."),
+                LOG.exception("Unable to create host : %s in K2.",
                               host_name)
-                raise exception.KaminarioCinderDriverException(
-                    reason=six.text_type(ex.message))
+                raise exception.KaminarioCinderDriverException(reason=ex)
         else:
             # Use existing host.
             LOG.debug("Use existing initiator hostname: %s in K2.", host_name)
@@ -160,11 +157,10 @@ class KaminarioFCDriver(common.KaminarioCinderDriver):
                 except Exception as ex:
                     if host_rs.total == 0:
                         self._delete_host_by_name(host_name)
-                    LOG.exception(_LE("Unable to add wwpn : %(wwpn)s to "
-                                      "host: %(host)s in K2."),
+                    LOG.exception("Unable to add wwpn : %(wwpn)s to "
+                                  "host: %(host)s in K2.",
                                   {'wwpn': wwpn, 'host': host_name})
-                    raise exception.KaminarioCinderDriverException(
-                        reason=six.text_type(ex.message))
+                    raise exception.KaminarioCinderDriverException(reason=ex)
         return host, host_rs, host_name
 
     def _build_initiator_target_map(self, connector, all_target_wwns):
